@@ -9,9 +9,19 @@
  *		modify it under the terms of the GNU General Public License
  *		as published by the Free Software Foundation; either version
  *		2 of the License, or (at your option) any later version.
+ *
+ *   jl 22.06.97  Logging connects and disconnects.
+ *   jl 23.06.97  Adjustable DTR droptime
+ *   jl 21.09.97  Conversion table filenames in dialdir
+ *   jl 05.10.97  Line speed changed to long in dial()
+ *   jl 26.01.98  last login date & time added to dialing window
+ *   jl 16.04.98  start searching for dialing tags from the highlighted entry
+ *
  */
+
 #include "port.h"
 #include "minicom.h"
+#include "intl.h"
 
 #if VC_MUSIC
 #  if defined(__linux__)
@@ -53,6 +63,21 @@ struct v1_dialent {
   struct dialent *next;
 };
 
+struct v3_dialent {
+  char name[32];
+  char number[32];
+  char script[32];
+  char username[32];
+  char password[32];
+  char term;
+  char baud[8];
+  char parity[2];
+  char dialtype;
+  char flags;
+  char bits[2];
+  struct dialent *next;
+};
+
 struct dialent {
   char name[32];
   char number[32];
@@ -65,6 +90,10 @@ struct dialent {
   char dialtype;
   char flags;
   char bits[2];
+  char lastdate[9];	/* jl 22.06.97 */
+  char lasttime[9];	/* jl 22.06.97 */
+  int  count;		/* jl 22.06.97 */
+  char convfile[16];	/* jl 21.09.97 */
   struct dialent *next;
 };
 
@@ -135,8 +164,8 @@ void modeminit()
 
   if (P_MINIT[0] == '\0') return;
 
-  w = tell("Initializing Modem");
-  m_dtrtoggle(portfd);
+  w = tell(_("Initializing Modem"));
+  m_dtrtoggle(portfd, 1);         /* jl 23.06.97 */
   mputs(P_MINIT, 0);
   wclose(w, 1);
 }
@@ -150,7 +179,7 @@ void modemreset()
 
   if (P_MRESET[0] == '\0') return;
 
-  w = tell("Resetting Modem");
+  w = tell(_("Resetting Modem"));
   mputs(P_MRESET, 0);
   sleep(1);
   wclose(w, 1);
@@ -162,11 +191,15 @@ void modemreset()
 void hangup()
 {
   WIN *w;
+  int sec=1;
 
-  w = tell("Hanging up");
+  w = tell(_("Hanging up"));
 
-  if (P_MDROPDTR[0] == 'Y') {
-  	m_dtrtoggle(portfd);
+  if (isdigit(P_MDROPDTR[0]))
+    sscanf(P_MDROPDTR,"%2d",&sec);
+
+  if (P_MDROPDTR[0] == 'Y' || (isdigit(P_MDROPDTR[0]) && sec>0)) {
+  	m_dtrtoggle(portfd, sec);   /* jl 23.06.97 */
   } else {
   	mputs(P_MHANGUP, 0);
   	sleep(1);
@@ -174,6 +207,7 @@ void hangup()
   /* If we don't have DCD support fake DCD dropped */
   bogus_dcd = 0;
   wclose(w, 1);
+  do_log(_("Hangup"));   /* jl 22.06.97 */
 }
 
 /*
@@ -184,7 +218,7 @@ void sendbreak()
 {
   WIN *w;
   
-  w = tell("Sending BREAK");
+  w = tell(_("Sending BREAK"));
   wcursor(w, CNONE);
 
   m_break(portfd);
@@ -247,14 +281,14 @@ int rtime;
   int ret = 0;
 
   wlocate(dialwin, 1, 5);
-  wprintf(dialwin, "    No connection: %s.      \n", s);
+  wprintf(dialwin, _("    No connection: %s.      \n"), s);
   if (rtime < 0) {
-  	wprintf(dialwin, "   Press any key to continue..    ");
+  	wprintf(dialwin, _("   Press any key to continue..    "));
 	if (check_io(-1, 0, 10000, NULL, NULL) & 2) 
 		(void) keyboard(KGETKEY, 0);
   	return(0);
   }
-  wprintf(dialwin, "     Retry in %2d seconds             ", rtime);
+  wprintf(dialwin, _("     Retry in %2d seconds             "), rtime);
   
   for(f = rtime - 1; f >= 0; f--) {
 	x = check_io(-1, 0, 1000, NULL, NULL);
@@ -264,8 +298,8 @@ int rtime;
 		if (x != ' ') ret = -1;
 		break;
 	}
-  	wlocate(dialwin, 14, 6);
-  	wprintf(dialwin, "%2d ", f);
+  	wlocate(dialwin, 0, 6);
+	wprintf(dialwin, _("     Retry in %2d seconds             "), f);
   }
 #ifdef __linux__
   /* MARK updated 02/17/94 - Min dial delay set to 0.35 sec instead of 1 sec */
@@ -274,33 +308,42 @@ int rtime;
   if (f < 0) sleep(1);
 #endif
   wlocate(dialwin, 1, 5);
-  wprintf(dialwin, "                              \n");
-  wprintf(dialwin, "                        ");
+  wprintf(dialwin, "                                       \n");
+  wprintf(dialwin, "                                             ");
   return(ret);
 }
 
 /*
  * Dial a number, and display the name.
  */
-int dial(d)
+long dial(d, d2)
 struct dialent *d;
+struct dialent **d2;
 {
   char *s = 0, *t = 0;
-  int f, x = 0, nb, retst = -1;
+  int f, x = 0;
   int modidx, retries = 0;
   int maxretries = 1, rdelay = 45;
-  char *reason = "Max retries";
+  long nb, retst = -1;
+  char *reason = _("Max retries");
   time_t now, last;
+  struct tm *ptime;
   char buf[128];
   char modbuf[128];
+  char logline[128];
 
-  dialwin = wopen(18, 9, 62, 15, BSINGLE, stdattr, mfcolor, mbcolor, 0, 0, 1);
-  wtitle(dialwin, TMID, "Autodial");
+  timer_update(); /* Statusline may still show 'Online' / jl 16.08.97 */
+  dialwin = wopen(18, 9, 62, 16, BSINGLE, stdattr, mfcolor, mbcolor, 0, 0, 1);
+  wtitle(dialwin, TMID, _("Autodial"));
   wcursor(dialwin, CNONE);
 
   wputs(dialwin, "\n");
-  wprintf(dialwin, " Dialing : %s\n", d->name);
-  wprintf(dialwin, "      At : %s\n", d->number);
+  wprintf(dialwin, " %s : %s\n", _("Dialing"), d->name);
+  wprintf(dialwin, _("      At : %s\n"), d->number);
+  if (d->lastdate[0] && d->lasttime[0])		/* jl 26.01.98 */
+    wprintf(dialwin, _(" Last on : %s at %s\n"), d->lastdate, d->lasttime);
+  else
+    wprintf(dialwin, "\n");
   wredraw(dialwin, 1);
 
   /* Tell keyboard routines we need them. */
@@ -322,10 +365,15 @@ MainLoop:
 			if (d == (struct dialent *)NULL) d = dialents;
 		} while(!(d->flags & FL_TAG));
 		wlocate(dialwin, 0, 1);
-		wprintf(dialwin, " Dialing : %s", d->name);
+		wprintf(dialwin, _(" Dialing : %s"), d->name);
 		wclreol(dialwin);
-		wprintf(dialwin, "\n      At : %s", d->number);
+		wprintf(dialwin, _("\n      At : %s"), d->number);
 		wclreol(dialwin);
+		if (d->lastdate[0] && d->lasttime[0]) {
+		  wprintf(dialwin, _("\n Last on : %s at %s"),
+			  d->lastdate, d->lasttime);
+		  wclreol(dialwin);
+		}
 	}
 
 	/* Calculate dial time */
@@ -335,10 +383,10 @@ MainLoop:
 	last = now;
 
   	/* Show used time */
-	wlocate(dialwin, 0, 3);
-	wprintf(dialwin, "    Time : %-3d", dialtime);
-	if (maxretries > 1) wprintf(dialwin, "     Attempt #%d", retries);
-	wputs(dialwin, "\n\n\n Escape to cancel, space to retry.");
+	wlocate(dialwin, 0, 4);
+	wprintf(dialwin, _("    Time : %-3d"), dialtime);
+	if (maxretries > 1) wprintf(dialwin, _("     Attempt #%d"), retries);
+	wputs(dialwin, _("\n\n\n Escape to cancel, space to retry."));
 	
 	/* Start the dial */
 	m_flush(portfd);
@@ -404,17 +452,35 @@ MainLoop:
 		    !strncmp(modbuf, P_MCONNECT, strlen(P_MCONNECT))) {
 			retst = 0;
 			/* Try to do auto-bauding */
-			if (sscanf(modbuf + strlen(P_MCONNECT), "%d", &nb) == 1)
+			if (sscanf(modbuf + strlen(P_MCONNECT), "%ld", &nb) == 1)
 				retst = nb;
 
 			/* Try to figure out if this system supports DCD */
 			f = m_getdcd(portfd);
 			bogus_dcd = 1;
 
-			wlocate(dialwin, 1, 6);
+			/* jl 22.05.97, 22.09.97 */
+			snprintf(logline, sizeof(logline), "%s %s, %s",
+				modbuf, d->name, d->number);
+			do_log(logline);
+
+			ptime=localtime(&now);
+			sprintf(d->lastdate,"%02d%02d%02d",
+				ptime->tm_year,(ptime->tm_mon)+1,
+				ptime->tm_mday);
+			sprintf(d->lasttime,"%02d:%02d",
+				ptime->tm_hour,ptime->tm_min);
+			d->count ++;
+
+			if (d->convfile[0]) {
+			  loadconv(d->convfile);    /* jl 21.09.97 */
+			  strcpy(P_CONVF, d->convfile);
+			}
+
+			wlocate(dialwin, 1, 7);
 			if (d->script[0] == 0) {
 				wputs(dialwin,
-				     "Connected. Press any key to continue");
+				     _("Connected. Press any key to continue"));
 #if VC_MUSIC
 				if (P_SOUND[0] == 'Y')
 					music();
@@ -441,6 +507,9 @@ MainLoop:
 
 			/* Un-tag this entry. */
 			d->flags &= ~FL_TAG;
+			/* store pointer to the entry that ANSWERED */
+			if (d2 != (struct dialent**)NULL)
+			  *d2 = d;	/* jl 23.09.97 */
 			return(retst);
 		}
 		for(f = 0; f < 3; f++) {
@@ -468,7 +537,7 @@ MainLoop:
 	    if (last != now) {
 		dialtime -= (now - last);
 		if (dialtime < 0) dialtime = 0;
-		wlocate(dialwin, 11, 3);
+		wlocate(dialwin, 11, 4);
 		wprintf(dialwin, "%-3d  ", dialtime);
 		if (dialtime <= 0) {
 			mputs(P_MDIALCAN, 0);
@@ -515,17 +584,47 @@ static struct dialent *mkstdent()
   strcpy(d->baud, "Curr");
   strcpy(d->bits, "8");
   strcpy(d->parity, "N");
+  d->lastdate[0] = 0;	/* jl 22.06.97 */
+  d->lasttime[0] = 0;
+  d->count = 0;
+  d->convfile[0] = 0;	/* jl 21.09.97 */
   d->next = (struct dialent *)0;
 
   return(d);
 }
 
+/* Read version 4 of the dialing directory. */
+void v4_read(fp, d, dv)
+struct dialent *d;
+FILE *fp;
+struct dver *dv;	/* jl 21.09.97 */
+{
+  (void) fread((char *)d, dv->size, (size_t)1, fp);
+
+  if (dv->size < sizeof(struct dialent)) {
+    d->convfile[0]=0;
+    if (dv->size < offsetof(struct dialent, count) + sizeof(struct dialent *)){
+      d->count = 0;
+      d->lasttime[0] = 0;
+      d->lastdate[0] = 0;
+    }
+  }
+}
+
 /* Read version 3 of the dialing directory. */
 void v3_read(fp, d)
 struct dialent *d;
 FILE *fp;
 {
-  (void) fread((char *)d, sizeof(struct dialent), (size_t)1, fp);
+  struct v3_dialent v3;   /* jl 22.06.97 */
+
+  (void) fread((char *)&v3, sizeof(v3), (size_t)1, fp);
+  memcpy(d, &v3, offsetof(struct v3_dialent, next));
+  
+  d->lastdate[0]=0;
+  d->lasttime[0]=0;
+  d->count=0;
+  d->convfile[0]=0;
 }
 
 /* Read version 2 of the dialing directory. */
@@ -533,8 +632,15 @@ void v2_read(fp, d)
 struct dialent *d;
 FILE *fp;
 {
-  (void) fread((char *)d, sizeof(struct dialent), (size_t)1, fp);
+  struct v3_dialent v3;   /* jl 22.06.97 */
+
+  (void) fread((char *)&v3, sizeof(v3), (size_t)1, fp);
+  memcpy(d, &v3, offsetof(struct v3_dialent, next));
   if (d->flags & FL_ANSI) d->flags |= FL_WRAP;
+  d->lastdate[0]=0;
+  d->lasttime[0]=0;
+  d->count=0;
+  d->convfile[0]=0;
 }
 
 /* Read version 1 of the dialing directory. */
@@ -546,9 +652,13 @@ struct dialent *d;
 
   fread((char *)&v1, sizeof(v1), (size_t)1, fp);
   memcpy(d->username, v1.username, sizeof(v1) - offsetof(struct v1_dialent, username));
-  strcpy(d->name, v1.name);
-  strcpy(d->number, v1.number);
-  strcpy(d->script, v1.script);
+  strncpy(d->name, v1.name, sizeof(d->name));
+  strncpy(d->number, v1.number, sizeof(d->number));
+  strncpy(d->script, v1.script, sizeof(d->script));
+  d->lastdate[0]=0;
+  d->lasttime[0]=0;
+  d->count=0;
+  d->convfile[0]=0;
 }
 
 /* Read version 0 of the dialing directory. */
@@ -585,12 +695,12 @@ int readdialdir()
   strcpy(d_man->name, "Manually entered number");
 
   /* Construct path */
-  sprintf(dfile, "%s/.dialdir", homedir);
+  snprintf(dfile, sizeof(dfile), "%s/.dialdir", homedir);
 
   /* Try to open ~/.dialdir */
   if ((fp = sfopen(dfile, "r")) == (FILE *)NULL) {
 	if (errno == EPERM) {
-		werror("Cannot open ~/.dialdir: permission denied");
+		werror(_("Cannot open ~/.dialdir: permission denied"));
 		dialents = mkstdent();
 		dendd = 1;
 		return(0);
@@ -626,17 +736,35 @@ int readdialdir()
 		break;
 	case 2:
 	case 3:
-		dial_ver.size = sizeof(struct dialent);
+		dial_ver.size = sizeof(struct v3_dialent);
+		break;
+	case 4:
+	  /* dial_ver.size = sizeof(struct dialent); */
+	  /* Removed the forced setting to add flexibility.
+	   * Now you don't need to change the version number
+	   * if you just add fields to the end of the dialent structure
+	   * before the *next pointer and don't change existing fields.
+	   * Just update the initialization in the functions
+	   * v4_read and mkstdent (and whatever you added the field for)
+	   * jl 21.09.97
+	   */
+	  	if (dial_ver.size < 200 ||
+		    dial_ver.size > sizeof(struct dialent)) {
+		  werror(_("Phonelist garbled (unknown version?)"));
+		  dendd = 1;
+		  dialents = mkstdent();
+		  return(-1);
+		}
 		break;
 	default:
-  		werror("Unknown dialing directory version");
+  		werror(_("Unknown dialing directory version"));
 		dendd = 1;
   		dialents = mkstdent();
   		return(-1);
   }
 
   if (size % dial_ver.size != 0) {
-  	werror("Phonelist garbled (?)");
+  	werror(_("Phonelist garbled (?)"));
   	fclose(fp);
 #if 0 /* This should be less facistic now. */
   	unlink(dfile);
@@ -663,7 +791,7 @@ int readdialdir()
 			else
 				prev->next = (struct dialent *)0;
 
-  			werror("Out of memory while reading dialing directory");
+  			werror(_("Out of memory while reading dialing directory"));
 			fclose(fp);
   			return(-1);
   	}
@@ -680,6 +808,9 @@ int readdialdir()
 		case 3:
 			v3_read(fp, d);
 			break;
+		case 4:
+			v4_read(fp, d, &dial_ver);
+			break;
 	}
 	/* MINIX terminal type is obsolete */
 	if (d->term == 2) d->term = 1;
@@ -710,12 +841,12 @@ static void writedialdir()
   /* Make no sense if access denied */
   if (dendd) return;
 
-  sprintf(dfile, "%s/.dialdir", homedir);
+  snprintf(dfile, sizeof(dfile), "%s/.dialdir", homedir);
 
   omask = umask(077);
   if ((fp = sfopen(dfile, "w")) == (FILE *)0) {
 	(void)umask(omask);
-  	werror("Can't write to ~/.dialdir");
+  	werror(_("Can't write to ~/.dialdir"));
 	dendd = 1;
   	return;
   }
@@ -724,8 +855,12 @@ static void writedialdir()
   d = dialents;
   /* Set up version info. */
   dial_ver.magic = DIALMAGIC;
-  dial_ver.version = 3;
+  dial_ver.version = 4;
   dial_ver.size = sizeof(struct dialent);
+  dial_ver.res1 = 0;	/* We don't use these res? fields, but let's */
+  dial_ver.res2 = 0;	/* initialize them to a known init value for */
+  dial_ver.res3 = 0;	/* whoever needs them later / jl 22.09.97    */
+  dial_ver.res4 = 0;
   fwrite(&dial_ver, sizeof(dial_ver), (size_t)1, fp);
 
   /* Write dialing directory */
@@ -733,7 +868,7 @@ static void writedialdir()
 	oldfl = d->flags;
 	d->flags &= FL_SAVE;
   	if (fwrite(d, sizeof(struct dialent), (size_t)1, fp) != 1) {
-  		werror("Error writing ~/.dialdir!");
+  		werror(_("Error writing ~/.dialdir!"));
   		fclose(fp);
   		return;
   	}
@@ -771,28 +906,44 @@ struct dialent *d;
 {
   WIN *w;
   int c;
+  char* name = _(" A -  Name                :"),
+      * number = _(" B -  Number              :"),
+      * dial_string = _(" C -  Dial string #       :"),
+      * local_echo = _(" D -  Local echo          :"),
+      * script = _(" E -  Script              :"),
+      * username = _(" F -  Username            :"),
+      * password = _(" G -  Password            :"),
+      * terminal_emulation = _(" H -  Terminal Emulation  :"),
+      * backspace_key_sends = _(" I -  Backspace key sends :"),
+      * linewrap = _(" J -  Linewrap            :"),
+      * line_settings = _(" K -  Line Settings       :"),
+      * conversion_table = _(" L -  Conversion table    :"),
+      * question = _("Change which setting?");
   
-  w = wopen(5, 5, 75, 17, BDOUBLE, stdattr, mfcolor, mbcolor, 0, 0, 1);
-  wprintf(w, " A -  Name                : %s\n", d->name);
-  wprintf(w, " B -  Number              : %s\n", d->number);
-  wprintf(w, " C -  Dial string #       : %d\n", d->dialtype + 1);
-  wprintf(w, " D -  Local echo          : %s\n", yesno(d->flags & FL_ECHO));
-  wprintf(w, " E -  Script              : %s\n", d->script);
-  wprintf(w, " F -  Username            : %s\n", d->username);
-  wprintf(w, " G -  Password            : %s\n", d->password);
-  wprintf(w, " H -  Terminal Emulation  : %s\n", te[d->term - 1]);
-  wprintf(w, " I -  Backspace key sends : %s\n",
-	d->flags & FL_DEL ? "Delete" : "Backspace");
-  wprintf(w, " J -  Linewrap            : %s\n",
-	d->flags & FL_WRAP ? "On" : "Off");
-  wprintf(w, " K -  Line Settings       : %s %s%s1",
+  w = wopen(5, 4, 75, 19, BDOUBLE, stdattr, mfcolor, mbcolor, 0, 0, 1);
+  wprintf(w, "%s %s\n", name, d->name);
+  wprintf(w, "%s %s\n", number, d->number);
+  wprintf(w, "%s %d\n", dial_string, d->dialtype + 1);
+  wprintf(w, "%s %s\n", local_echo, _(yesno(d->flags & FL_ECHO)));
+  wprintf(w, "%s %s\n", script, d->script);
+  wprintf(w, "%s %s\n", username, d->username);
+  wprintf(w, "%s %s\n", password, d->password);
+  wprintf(w, "%s %s\n", terminal_emulation, te[d->term - 1]);
+  wprintf(w, "%s %s\n", backspace_key_sends,
+	d->flags & FL_DEL ? _("Delete") : _("Backspace"));
+  wprintf(w, "%s %s\n", linewrap,
+	d->flags & FL_WRAP ? _("On") : _("Off"));
+  wprintf(w, "%s %s %s%s1\n", line_settings,
   	d->baud, d->bits, d->parity);
-  wlocate(w, 4, 13);
-  wputs(w, "Change which setting? ");
+  wprintf(w, "%s %s\n", conversion_table, d->convfile);
+  wprintf(w, _("      Last dialed         : %s %s\n"),d->lastdate,d->lasttime);
+  wprintf(w, _("      Times on            : %d"),d->count);
+  wlocate(w, 4, 15);
+  wputs(w, question);
   wredraw(w, 1);
 
   while(1) {
-      wlocate(w, 26, 13);
+      wlocate(w, strlen (question) + 5, 15);
       c = wxgetch();
       if (c >= 'a') c -= 32;
       switch(c) {
@@ -802,42 +953,42 @@ struct dialent *d;
   		wclose(w, 1);
   		return;
   	case 'A':
-  		wlocate(w, 28, 0);
+  		wlocate(w, strlen (name) + 1, 0);
   		(void) wgets(w, d->name, 31, 32);
   		break;
   	case 'B':
-  		wlocate(w, 28, 1);
+  		wlocate(w, strlen (number) + 1, 1);
   		(void) wgets(w, d->number, 31, 32);
   		break;
 	case 'C':
 		d->dialtype = (d->dialtype + 1) % 3;
-		wlocate(w, 28, 2);
+		wlocate(w, strlen (dial_string) + 1, 2);
 		wprintf(w, "%d", d->dialtype + 1);
 		wflush();
 		break;
 	case 'D':
 		d->flags ^= FL_ECHO;
-		wlocate(w, 28, 3);
-		wprintf(w, "%s", yesno(d->flags & FL_ECHO));
+		wlocate(w, strlen (local_echo) + 1, 3);
+		wprintf(w, "%s", _(yesno(d->flags & FL_ECHO)));
 		wflush();
 		break;
   	case 'E':
-  		wlocate(w, 28, 4);
+  		wlocate(w, strlen (script) + 1, 4);
   		(void) wgets(w, d->script, 31, 32);	
   		break;
   	case 'F':
-  		wlocate(w, 28, 5);
+  		wlocate(w, strlen (username) + 1, 5);
   		(void) wgets(w, d->username, 31, 32);
   		break;
   	case 'G':
-  		wlocate(w, 28, 6);
+  		wlocate(w, strlen (password) + 1, 6);
   		(void) wgets(w, d->password, 31, 32);
   		break;	
   	case 'H':
   		d->term = (d->term % 3) + 1;
 		/* MINIX == 2 is obsolete. */
 		if (d->term == 2) d->term = 3;
-  		wlocate(w, 28, 7);
+  		wlocate(w, strlen (terminal_emulation) + 1, 7);
   		wputs(w, te[d->term - 1]);	
 
 		/* Also set backspace key. */
@@ -848,26 +999,30 @@ struct dialent *d;
 			d->flags &= ~FL_WRAP;
 			d->flags |= FL_DEL;
 		}
-		wlocate(w, 28, 8);
-		wputs(w, d->flags & FL_DEL ? "Delete   " : "Backspace");
-		wlocate(w, 28, 9);
-		wputs(w, d->flags & FL_WRAP ? "On " : "Off");
+		wlocate(w, strlen (backspace_key_sends) + 1, 8);
+		wputs(w, d->flags & FL_DEL ? _("Delete   ") : _("Backspace"));
+		wlocate(w, strlen (linewrap) + 1, 9);
+		wputs(w, d->flags & FL_WRAP ? _("On ") : _("Off"));
   		break;
 	case 'I':
 		d->flags ^= FL_DEL;
-		wlocate(w, 28, 8);
-		wputs(w, d->flags & FL_DEL ? "Delete   " : "Backspace");
+		wlocate(w, strlen (backspace_key_sends) + 1, 8);
+		wputs(w, d->flags & FL_DEL ? _("Delete   ") : _("Backspace"));
 		break;
 	case 'J':
 		d->flags ^= FL_WRAP;
-		wlocate(w, 28, 9);
-		wputs(w, d->flags & FL_WRAP ? "On " : "Off");
+		wlocate(w, strlen (linewrap) + 1, 9);
+		wputs(w, d->flags & FL_WRAP ? _("On ") : _("Off"));
 		break;
   	case 'K':
   		get_bbp(d->baud, d->bits, d->parity, 1);
-  		wlocate(w, 28, 10);
+  		wlocate(w, strlen (line_settings) + 1, 10);
   		wprintf(w, "%s %s%s1  ", d->baud, d->bits, d->parity);
   		break;
+ 	case 'L':	/* jl 21.09.97 */
+		wlocate(w, strlen (conversion_table) + 1, 11);
+		(void) wgets(w, d->convfile, 15, 16);
+		break;
   	default:
   		break;
       }
@@ -875,38 +1030,49 @@ struct dialent *d;
 }
 
 static WIN *dsub;
-static char *what =  "  Dial    Find    Add     Edit   Remove  Manual ";
+static char *what =  N_("  Dial    Find    Add     Edit   Remove  Manual ");
+/*                       123456789012345678901234567890123456789012345678
+			          1         2         3         4            */
 static int dprev;
 
 /*
  * Highlight a choice in the horizontal menu.
  */
-static void dhili(k)
-int k;
+static void dhili(position_dialing_directory, k)
+int position_dialing_directory, k;
 {
+  /* acme@conectiva.com.br 28/02/1998 
+     hack to make i18n strings work with _this_ hack ;> :
+     the i18n string must have a options with the same size (padded 
+     with spaces... I only hope the translations are smart enough to
+     figure this out... ;> */
+
+  int option_size = strlen (_(what)) / 6;
+
   if (k == dprev) return;
 
   if (dprev >= 0) {
-  	wlocate(dsub, 14 + 8*dprev, 0);
+  	wlocate(dsub, position_dialing_directory + option_size * dprev, 0);
 	if (!useattr) {
 		wputs(dsub, " ");
 	} else {
+
   		wsetattr(dsub, XA_REVERSE | stdattr);
-  		wprintf(dsub, "%8.8s", what + 8*dprev);
+  		wprintf(dsub, "%*.*s", option_size, option_size, _(what) + option_size * dprev);
 	}
   }
   dprev = k;
-  wlocate(dsub, 14 + 8*k, 0);
+  wlocate(dsub, position_dialing_directory + option_size * k, 0);
   if (!useattr) {
 	wputs(dsub, ">");
   } else {
 	wsetattr(dsub, stdattr);
-	wprintf(dsub, "%8.8s", what + 8*k);
+	wprintf(dsub, "%*.*s", option_size, option_size, _(what) + option_size * k);
   }
 }
 
 
-static char *fmt = "\r %2d %c %-16.16s %-16.16s%5s %s%s1    %-6.6s %-15.15s\n";
+static char *fmt = "\r %2d %c%-16.16s%-16.16s%6s %5s %4d %-15.15s\n";
 
 /*
  * Print the dialing directory. Only draw from "cur" to bottom.
@@ -924,22 +1090,23 @@ int top, cur;
   for(f = start; f < dialw->ys - 2; f++) {
   	d = getno(f + top);
   	if (d == (struct dialent *)0) break;
-  	wprintf(dialw, fmt, f+1+top, (d->flags & FL_TAG) ? 16 : ' ',
-		d->name, d->number,
-  		d->baud, d->bits, d->parity, te[d->term - 1], d->script);
+  	wprintf(dialw, fmt, f+1+top, (d->flags & FL_TAG) ? '>' : ' ',
+		d->name, d->number, d->lastdate, d->lasttime, 
+  		d->count, d->script);
   }
   dirflush = 1;
   wflush();
 }
 
 /* Little menu. */
-static char *d_yesno[] = { "   Yes  ", "   No   ", CNULL };
+static char *d_yesno[] = { N_("   Yes  "), N_("   No   "), CNULL };
 
 /* Try to dial an entry. */
 static void dial_entry(d)
 struct dialent *d;
 {
-  int nb;
+  long nb;
+  struct dialent *d2;
 
   /* Change settings for this entry. */
   if (atoi(d->baud) != 0) {
@@ -962,15 +1129,21 @@ struct dialent *d;
   strcpy(P_BACKSPACE, d->flags & FL_DEL ? "DEL" : "BS");
 
   /* Now that everything has been set, dial. */
-  if ((nb = dial(d)) < 0) return;
+  if ((nb = dial(d, &d2)) < 0) return;
+  
+  if (d2 != (struct dialent *)NULL)
+    d = d2;	/* jl 22.09.97 */
 
   /* Did we detect a baudrate , and can we set it? */
   if (P_MAUTOBAUD[0] == 'Y' && nb) {
-	sprintf(P_BAUDRATE, "%d", nb);
+	sprintf(P_BAUDRATE, "%ld", nb);
 	port_init();
 	mode_status();
   }
 
+  /* Make sure the last date is updated   / jl 22.06.97 */
+  writedialdir();
+
   /* Run script if needed. */
   if (d->script[0]) runscript(0, d->script, d->username, d->password);
 
@@ -1000,7 +1173,7 @@ char *entry;
 
   /* Not found. */
   if (d == NULL) {
-	sprintf(buf, "Entry \"%s\" not found. Enter dialdir?", entry);
+	snprintf(buf, sizeof(buf), _("Entry \"%s\" not found. Enter dialdir?"), entry);
 	if (ask(buf, d_yesno) != 0) return;
 	dialdir();
 	return;
@@ -1029,6 +1202,7 @@ void dialdir()
   char *s, dname[64];
   static char manual[64];
   int changed = 0;
+  int position_dialing_directory = ((COLS / 2) + 32 - strlen (_(what))) / 2;
 
   dprev = -1;
   dname[0] = 0;
@@ -1042,18 +1216,19 @@ void dialdir()
   		XA_REVERSE | stdattr, mfcolor, mbcolor, 0, 0, 1);
   w = wopen(x1, 2, x2, LINES - 6, BSINGLE, stdattr, mfcolor, mbcolor, 0, 0, 1);
   wcursor(w, CNONE);
-  wtitle(w, TMID, "Dialing Directory");
-  wputs(w, "     Name               Number       Line Format Terminal Script\n");
-  wlocate(dsub, 14, 0);
-  wputs(dsub, what);
+  wtitle(w, TMID, _("Dialing Directory"));
+  wputs(w,
+	_("     Name            Number          Last on      Times Script\n"));
+  wlocate(dsub, position_dialing_directory, 0);
+  wputs(dsub, _(what));
 
   wsetregion(w, 1, w->ys - 1);
   w->doscroll = 0;
 
   prdir(w, top, top);  
-  wlocate(w, 14, w->ys - 1);
-  wputs(w, "( Escape to exit, Space to tag )");
-  dhili(subm);
+  wlocate(w, position_dialing_directory, w->ys - 1);
+  wputs(w, _("( Escape to exit, Space to tag )"));
+  dhili(position_dialing_directory, subm);
   dirflush = 1;
   wredraw(dsub, 1);
 
@@ -1121,7 +1296,7 @@ again:
   			wlocate(w, 4, cur + 1 - top);
 			d->flags ^= FL_TAG;
 			wsetattr(w, XA_REVERSE | stdattr);
-			wprintf(w, "%c", d->flags & FL_TAG ? 16 : ' ');
+			wprintf(w, "%c", d->flags & FL_TAG ? '>' : ' ');
 			wsetattr(w, XA_NORMAL | stdattr);
   			cur += (cur < nrents - 1);
 			break;
@@ -1146,7 +1321,7 @@ again:
 	}
   	if (cur != ocur) wcurbar(w, cur + 1 - top, XA_REVERSE | stdattr);
   	ocur = cur;
-  	dhili(subm);
+  	dhili(position_dialing_directory, subm);
   }
   quit = 0;
   /* ESC means quit */
@@ -1191,11 +1366,20 @@ again:
 
 	/* See if any entries were tagged. */
 	if (!(d->flags & FL_TAG)) {
-		for(d1 = dialents; d1; d1 = d1->next)
-			if (d1->flags & FL_TAG) {
-				d = d1;
-				break;
-			}
+	  /* First check the entries from the highlighted one to end.. */
+	  for (d1 = d; d1; d1 = d1->next)
+	    if (d1->flags & FL_TAG) {
+	      d = d1;
+	      break;
+	    }
+	  /* ..and if none of them was tagged, check from the begining. */
+	  if (!d1)
+	    for(d1 = dialents; d1 && d1!=d; d1 = d1->next)
+	      if (d1->flags & FL_TAG) {
+		d = d1;
+		break;
+	      }
+	  /* If no tags were found, we'll dial the highlighted one */
 	}
 	dial_entry(d);
 	return;
@@ -1256,11 +1440,11 @@ again:
 	changed++;
   	wlocate(w, 0, cur + 1 - top);
   	wprintf(w, fmt, cur+1, (d->flags & FL_TAG) ? 16 : ' ', d->name,
-	d->number, d->baud, d->bits, d->parity, te[d->term - 1], d->script);
+	d->number, d->lastdate, d->lasttime, d->count, d->script);
   }
   
   /* Delete an entry from the list */
-  if (subm == 4 && ask("Remove entry?", d_yesno) == 0) {
+  if (subm == 4 && ask(_("Remove entry?"), d_yesno) == 0) {
 	changed++;
   	if (nrents == 1) {
   		free((char *)d);
@@ -1297,7 +1481,7 @@ again:
 		wclose(dsub, 1);
 		
 		strcpy(d_man->number, manual);
-  		(void) dial(d_man);
+  		(void) dial(d_man, (struct dialent**)NULL);
   		return;
 	}
   }
