@@ -10,11 +10,13 @@
  *		2 of the License, or (at your option) any later version.
  *
  * fmg 1/11/94 color mods
+ * jl  22.06.97 log it when DCD drops
  *
  */
 #include "port.h"
 #include "minicom.h"
 #include "patchlevel.h"
+#include "intl.h"
 #if defined(__linux__) || (defined(BSD) && (BSD >= 199306))
 #include <errno.h>
 #endif
@@ -107,7 +109,7 @@ int doinit;
 
   /* First see if the lock file directory is present. */
   if (P_LOCK[0] && stat(P_LOCK, &stt) == 0)
-  	sprintf(lockfile, "%s/LCK..%s", P_LOCK, mbasename(dial_tty));
+  	snprintf(lockfile, sizeof(lockfile), "%s/LCK..%s", P_LOCK, mbasename(dial_tty));
   else
 	lockfile[0] = 0;
 
@@ -126,7 +128,7 @@ int doinit;
 		}
 		if (pid > 0 && kill((pid_t)pid, 0) < 0 &&
 			errno == ESRCH) {
-		    fprintf(stderr, "Lockfile is stale. Overriding it..\n");
+		    fprintf(stderr, _("Lockfile is stale. Overriding it..\n"));
 		    sleep(1);
 		    unlink(lockfile);
 		} else
@@ -134,7 +136,7 @@ int doinit;
 	}
 	if (n == 0) {
   		if (stdwin != NIL_WIN) wclose(stdwin, 1);
-  		fprintf(stderr, "Device %s is locked.\n", dial_tty);
+  		fprintf(stderr, _("Device %s is locked.\n"), dial_tty);
 		drop_privs();
 		return(-1);
 	}
@@ -144,7 +146,7 @@ int doinit;
   if (doinit > 0 && P_CALLOUT[0]) {
 	if(fastsystem(P_CALLOUT, CNULL, CNULL, CNULL) < 0) {
   		if (stdwin != NIL_WIN) wclose(stdwin, 1);
-  		fprintf(stderr, "Could not setup for dial out.\n");
+  		fprintf(stderr, _("Could not setup for dial out.\n"));
   		if (lockfile[0]) unlink(lockfile);
 		drop_privs();
 		return(-1);
@@ -180,15 +182,15 @@ int doinit;
 	if (doinit >= 0) {
   		if (stdwin != NIL_WIN) wclose(stdwin, 1);
 #if defined(__linux__) || (defined(BSD) && (BSD >= 199306))
-		fprintf(stderr, "minicom: cannot open %s: %s\n",
+		fprintf(stderr, _("minicom: cannot open %s: %s\n"),
 			dial_tty, strerror(s_errno));
 #else
-  		fprintf(stderr, "minicom: cannot open %s. Sorry.\n", dial_tty);
+  		fprintf(stderr, _("minicom: cannot open %s. Sorry.\n"), dial_tty);
 #endif
 		drop_privs();
 		return(-1);
 	}
-	werror("Cannot open %s!", dial_tty);
+	werror(_("Cannot open %s!"), dial_tty);
 	drop_privs();
 	return(-1);
   }
@@ -202,13 +204,13 @@ int doinit;
   	if ((fd = open(lockfile, O_WRONLY | O_CREAT | O_EXCL, 0666)) < 0) {
 #endif
   		if (stdwin != NIL_WIN) wclose(stdwin, 1);
-  		fprintf(stderr, "Cannot create lockfile. Sorry.\n");
+  		fprintf(stderr, _("Cannot create lockfile. Sorry.\n"));
 		drop_privs();
 		return(-1);
   	}
 	(void) umask(mask);
 	(void) chown(lockfile, (uid_t)real_uid, (gid_t)real_gid);
-  	sprintf(buf, "%10ld minicom %.20s\n", (long)getpid(), username);
+  	snprintf(buf, sizeof(buf), "%10ld minicom %.20s\n", (long)getpid(), username);
   	write(fd, buf, strlen(buf));
   	close(fd);
   }
@@ -357,11 +359,11 @@ void mode_status()
  */
 void time_status()
 {
-  wlocate(st, 66, 0);
+  wlocate(st, 63, 0);
   if (online < 0)
-  	wputs(st, P_HASDCD[0] == 'Y' ? " Offline     " : " OFFLINE     ");
+  	wprintf(st, " %12.12s ", P_HASDCD[0] == 'Y' ? _("Offline") : _("OFFLINE"));
   else
-  	wprintf(st," %s %02ld:%02ld", P_HASDCD[0] == 'Y' ? "Online" : "ONLINE",
+  	wprintf(st," %s %02ld:%02ld", P_HASDCD[0] == 'Y' ? _("Online") : _("ONLINE"),
   		online / 3600, (online / 60) % 60);
   		
   ret_csr();
@@ -382,9 +384,11 @@ void curs_status()
  */
 static void updtime()
 {
-  static int old_online = 0;
+  static int old_online = -1;
 
   if (old_online == online) return;
+  if ((old_online >= 0) && (online < 0))
+	do_log(_("Gone offline"));  /* jl 22.06.97 */
   old_online = online;
   if (st != NIL_WIN) {
   	time_status();
@@ -444,12 +448,12 @@ void show_status()
 {
   st->direct = 0;
   wlocate(st, 0, 0);
-  wprintf(st, " %7.7sZ for help \263           \263     \263 Minicom %-9.9s \263       \263 ",
+  wprintf(st, _(" %7.7sZ for help \263           \263     \263 Minicom %-6.6s \263       \263 "),
   	esc_key(), ST_VERSION);
   mode_status();
   time_status();
   curs_status();
-  wlocate(st, 59, 0);
+  wlocate(st, 56, 0);
   switch(terminal) {
   	case VT100:
   		wputs(st, "VT102");
@@ -471,9 +475,9 @@ char *s;
   if (st == NIL_WIN) return;
   wlocate(st, 39, 0);
   if (*s == 0)
-  	wprintf(st, "Minicom %-9.9s", ST_VERSION);
+  	wprintf(st, "Minicom %-6.6s", ST_VERSION);
   else
-  	wprintf(st, "script %-10.10s", s);
+      	wprintf(st, "script %-7.7s", s);
   ret_csr();
 }
 
@@ -542,10 +546,10 @@ dirty_goto:
 		st = NIL_WIN;
 		wclose(stdwin, 0);
                 if (win_init(tfcolor, tbcolor, XA_NORMAL) < 0)
-			leave("Could not re-initialize window system.");
+			leave(_("Could not re-initialize window system."));
 		init_emul(terminal, 0);
 #else
-		werror("Resize not supported, screen may be messed up!");
+		werror(_("Resize not supported, screen may be messed up!"));
 #endif
 	}
 	/* Update the timer. */
@@ -586,7 +590,11 @@ dirty_goto:
 		if (c < 0) c += 256; /* XXX - shouldn't happen */
 
 		/* Was this a command key? */
+#ifndef RH_FIX
 		if ((escape == 128 && c > 224 && c < 252) ||
+#else   /* filipg 8/22/97: needed to recognize Fkeys in RedHat 4.1 */
+		if ((escape == 128 && c > 128 && c < 256) ||
+#endif /* RH_FIX */
 		    (escape != 27 && c == escape) || 
 		    (c > K_META)) {
 
