@@ -11,19 +11,24 @@
  *		2 of the License, or (at your option) any later version.
  *
  * fmg 1/11/94 colors
- *
+ * jl  30.06.97 log it if you quit without reset while online
+ * fmg 8/20/97 History buffer searching added
+ * jseymour@jimsun.LinxNet.com (Jim Seymour) 03/26/98 - Added support for
+ *    multiple tty devices via new "get_port()" function.
  */
+
 #define EXTERN
 #include "port.h"
 #include "minicom.h"
 #include "patchlevel.h"
+#include "intl.h"
 
 #define RESET 1
 #define NORESET 2
 
 #ifdef _SVR2
 extern struct passwd *getpwuid();
-#endif
+#endif /*_SVR2*/
 
 #ifdef DEBUG
 /* Show signals when debug is on. */
@@ -31,20 +36,20 @@ static void signore(sig)
 int sig;
 {
   if (stdwin != NIL_WIN)
-	werror("Got signal %d", sig);
+	werror(_("Got signal %d"), sig);
   else
-	printf("Got signal %d\r\n", sig);
+	printf("%s\r\n", _("Got signal %d"), sig);
 }
-#endif
+#endif /*DEBUG*/
 
 /*
  * Sub - menu's.
  */
 
-static char *c1[] = { "   Yes  ", "   No   ", CNULL };
-static char *c2[] = { "  Close ", " Pause  ", "  Exit  ", CNULL };
-static char *c3[] = { "  Close ", " Unpause", "  Exit  ", CNULL };
-static char *c7[] = { "   Yes  ", "   No   ", CNULL };
+static char *c1[] = { N_("   Yes  "), N_("   No   "), CNULL };
+static char *c2[] = { N_("  Close "), N_(" Pause  "), N_("  Exit  "), CNULL };
+static char *c3[] = { N_("  Close "), N_(" Unpause"), N_("  Exit  "), CNULL };
+static char *c7[] = { N_("   Yes  "), N_("   No   "), CNULL };
 
 /* Initialize modem port. */
 void port_init()
@@ -58,7 +63,7 @@ int askit;
 {
   int c = 0;
  
-  if (askit) c = ask("Hang-up line?", c7);
+  if (askit) c = ask(_("Hang-up line?"), c7);
   if (c == 0) hangup();
 }
 
@@ -69,7 +74,7 @@ static void hangsig(sig)
 int sig;
 {
   if (stdwin != NIL_WIN)
-	werror("Killed by signal %d !\n", sig);
+	werror(_("Killed by signal %d !\n"), sig);
   if (capfp != (FILE *)0) fclose(capfp);
   keyboard(KUNINSTALL, 0);
   hangup();
@@ -89,7 +94,7 @@ int sig;
 
   wleave();
   signal(SIGTSTP, SIG_DFL);
-  printf("Suspended. Type \"fg\" to resume.\n");
+  printf(_("Suspended. Type \"fg\" to resume.\n"));
   kill(getpid(), SIGTSTP);
   signal(SIGTSTP, shjump);
   wreturn();
@@ -109,11 +114,11 @@ int dummy;
 
   sh = getenv("SHELL");
   if (sh == CNULL) {
-  	werror("SHELL variable not set");
+  	werror(_("SHELL variable not set"));
   	return;
   }
   if ((pid = fork()) == -1) {
-  	werror("Out of memory: could not fork()");
+  	werror(_("Out of memory: could not fork()"));
   	return;
   }
   if (pid != 0) wleave();
@@ -123,7 +128,7 @@ int dummy;
 	set_privs();
 	setgid((gid_t)real_gid);
   	setuid((uid_t)real_uid);
-	printf("Shelled out. Type \"exit\" to return.\n");
+	printf(_("Shelled out. Type \"exit\" to return.\n"));
   	execl(sh, sh, CNULL);
   	exit(1);
   }
@@ -131,9 +136,9 @@ int dummy;
   wreturn();
   if (use_status) show_status();
 }
-#endif 
+#endif /*SIGTSTP*/
 
-#if HISTORY
+#if _HISTORY
 /* Get a line from either window or scroll back buffer. */
 static ELM *getline(w, no)
 WIN *w;
@@ -170,16 +175,201 @@ int r;
   w->direct = 1;
 }
 
-/* Scroll back */
-static void scrollback()
+#if _SEARCH_HISTORY
+/*
+ * fmg 8/20/97
+ * drawhist_look()
+ * Redraw the window, highlight line that was found to contain
+ * pattern 'look'
+ * Needed by re-draw screen function after EACH find_next()
+ */
+void drawhist_look(w, y, r, look, case_matters)
+WIN	*w;
+int	y;
+int	r;
+char	*look;
+int	case_matters;
+{
+  int f;
+  ELM *tmp_e;
+  char tmp_line[256];	/* should be enough */
+
+  tmp_line[0]='\0';
+  w->direct = 0;
+  for (f = 0; f < w->ys; f++)
+  {
+	tmp_e = getline(w, y++);
+
+  	/* First we "accumulate" the line into a variable */
+	wdrawelm_var(w, f, tmp_e, tmp_line);
+
+	/* Does it have what we want? */
+	if (strlen(look)>1 && strlen(tmp_line)>1) {
+		if (StrStr(tmp_line,look, case_matters))
+			wdrawelm_inverse(w, f, tmp_e); /* 'inverse' it */
+  		else
+			wdrawelm(w, f, tmp_e); /* 'normal' output */
+	}
+  }
+
+  if (r) wredraw(w, 1);
+  w->direct = 1;
+}
+
+/*
+ * fmg 8/20/97
+ * Search history - main function that started the C-code blasphemy :-)
+ * This function doesn't care about case/case-less status...
+ */
+void searchhist(w_orig, w_hist, r, str)
+WIN *w_orig,*w_hist;
+int r;
+char *str;
 {
   int y;
-  WIN *b_us, *b_st;
-  int c;
+  WIN *w_new;
   char hline[128];
 
+  /* Find out how big a window we must open. */
+  y = w_hist->y2;
+  if (st == (WIN *)0 || (st && tempst)) y--;
+
+  /* Open a Search line window. */
+  w_new = wopen(0, y+1, w_hist->x2, y+1, 0, st_attr, sfcolor, sbcolor, 0, 0, 1);
+  w_new->doscroll = 0;
+  w_new->wrap = 0;
+
+  strcpy(hline, _("SEARCH FOR (ESC=Exit)"));
+  wprintf(w_new, "%s(%d):",hline,MAX_SEARCH);
+  wredraw(w_new, 1);
+  wflush();
+
+  wlocate(w_new, strlen(hline)+6, y+1);
+  (void) wgets(w_new, str, MAX_SEARCH, MAX_SEARCH);
+  if (!str[0]) /* then unchanged... must have pressed ESC... get out */
+  {
+  	wflush();
+  	wclose(w_new, 0);
+  	return;
+  }
+
+  wredraw(w_hist, 1);
+  wflush();
+  wclose(w_new, 0);
+
+  return;
+}
+
+/*
+ * fmg 8/20/97
+ * Move scope to next hit of pattern in the buffer.
+ * Returns line-number of next "hit_line" or -1 if none found (we beep elsewhere ;-)
+ */
+int find_next(w, hit_line, look, case_matters)
+WIN	*w;
+int	hit_line;	/* 'current' Match line */
+char	*look;		/* pattern */
+int	case_matters;	/* guess... */
+{
+  int next_line;
+  ELM *tmp_e;
+  char tmp_line[256];	/* Should be more than enough... */
+
+  if (!look)
+  	return(++hit_line); /* next line */
+
+  tmp_line[0]='\0';	/* Personal phobia, I need to do this.. */
+
+  hit_line++;	/* we NEED this so we don't search only same line! */
+
+  if (hit_line >= us->histlines)	/* Make sure we've got a valid line! */
+  {
+	werror(_("Search Wrapping Around to Start!"));
+	hit_line = 0;
+  }
+
+  for (next_line = hit_line; next_line <= us->histlines; next_line++)
+  {
+	/* we do 'something' here... :-) */
+	tmp_e = getline(w, next_line);
+
+  	/*
+	 * First we "accumulate" the line into a variable.
+	 * To see 'why', see what an 'ELM' structure looks like!
+	 */
+	wdrawelm_var(w, next_line, tmp_e, tmp_line);
+
+	/* Does it have what we want? */
+	if (strlen(tmp_line) > 1 && strlen(look) > 1)
+		if (StrStr(tmp_line, look, case_matters))
+  			return(next_line);
+  }
+
+  if (hit_line >= us->histlines)	/* Make sure we've got a valid line! */
+  {
+	werror(_("Search Wrapping Around to Start!"));
+	hit_line = 0;
+  }
+
+  return(-1); /* nothing found! */
+}
+
+/*
+ * fmg 8/22/97
+ * Needed this for the case-less conparison... and Linux libc
+ * doesn't seem to have a strnstr function... so we fudge.. ;-)
+ */
+const char *upcase(dest, src)
+char	*dest, *src;
+{
+char	*d;
+
+d=dest;
+while (*src)
+	*d++=toupper(*src++);
+*d='\0';
+return (dest);
+}
+
+/*
+ * fmg 8/22/97
+ * Needed this for the case-less conparison... and Linux libc
+ * doesn't seem to have a strnstr function... so we fudge.. ;-)
+ */
+char *StrStr(str1, str2, case_matters)
+char	*str1, *str2;
+int	case_matters;
+{
+char	tmpstr1[256],
+	tmpstr2[256];
+
+if (case_matters)
+	return (strstr(str1,str2));
+else
+  	return (strstr(upcase(tmpstr1,str1),upcase(tmpstr2,str2)));
+}
+
+
+#endif /*_SEARCH_HISTORY*/
+
+/* Scroll back */
+static void scrollback()
+{
+  int y,c;
+  WIN *b_us, *b_st;
+#if _SEARCH_HISTORY
+  ELM *tmp_e;
+  int case_matters=0;	/* fmg: case-importance, needed for 'N' */
+  static char look_for[MAX_SEARCH];	/* fmg: last used search pattern */
+  char tmp_line[256];
+#endif /*SEARCH_HISTORY*/
+
+  char hline[128];
+  static int hit=0;
+
   /* Find out how big a window we must open. */
   y = us->y2;
+
   if (st == (WIN *)0 || (st && tempst)) y--;
 
   /* Open a window. */ 
@@ -193,8 +383,20 @@ static void scrollback()
   b_st->wrap = 0;
 
   /* Make sure help line is as wide as window. */
-  strcpy(hline, "   SCROLL MODE    U=up D=down F=page-forward B=page-backward ESC=exit                    ");
-  if (b_st->xs < 127) hline[b_st->xs] = 0;
+
+#if _SEARCH_HISTORY
+  /*
+   * fmg 8/20/97
+   * added /=Srch, \=CaseLess, and N=Next and changed rest of line...
+   * Hope you like it :-)
+   */
+  strcpy(hline, _("HISTORY: U=Up1 D=Down1 F=PgDn B=PgUp s=Srch S=CaseLess N=NextHit ESC=Exit "));
+#else
+  strcpy(hline, "   SCROLL MODE    U=up D=down F=page-forward B=page-backward ESC=exit       ");
+#endif /*_HIST_SEARCH*/
+
+  if (b_st->xs < 127)
+  	hline[b_st->xs] = 0;
   wprintf(b_st, hline);
   wredraw(b_st, 1);
   wflush();
@@ -202,39 +404,194 @@ static void scrollback()
   /* And do the job. */
   y = us->histlines;
 
+  /* fmg 8/20/97
+   * Needed for N)extSearch, keeps track of line on which current "hit"
+   * is... we advance it to 'N'ext hit in find_next(). We start at "top"
+   * of history stack
+   */
+  hit=0;
+  
   drawhist(b_us, y, 0);
 
   while((c = wxgetch()) != K_ESC) {
 	switch(c) {
+#if _SEARCH_HISTORY
+	  /*  
+	   * fmg 8/22/97
+	   * Take care of the search key: Caseless
+	   */
+	  case '\\':
+	  case 'S':
+	    case_matters=0; /* case-importance, ie. none :-) */
+	  /* 
+	   * fmg 8/22/97
+	   * Take care of the search key: Exact Match
+	   */
+	  case '/':
+	  case 's':
+  		if (!us->histlines)
+		{ wbell(); werror(_("History buffer Disabled!")); break; }
+  		if (!us->histline)
+		{ wbell(); werror(_("History buffer empty!")); break; }
+		
+		/* we need this for the case-importance-toggle to work.. */
+		if (c == '/' || c == 's')
+			case_matters=1; /* case-importance, ie. DOES */
+
+		/* open up new search window... */
+		searchhist(us, b_us, 0, look_for);
+		/* must redraw status line... */
+  		wlocate(b_st, 0, 0); /* move back to column 0! */
+  		wprintf(b_st, hline); /* and show the above-defined hline */
+  		wredraw(b_st, 1); /* again... */
+		/* highlight any matches */
+  		if (strlen(look_for)>1)
+		{
+  			hit = find_next(us, hit, look_for, case_matters);
+
+			if (hit == -1) { wbell(); wflush(); hit=0; break; }
+  			drawhist_look(b_us, hit, 1, look_for, case_matters);
+		}
+  		else
+		{ wbell(); break; }
+  		wflush();
+		break;
+	  /* 
+	   * fmg 8/22/97
+	   * Take care of the Next Hit key...
+	   * Popup an error window if no previous... why not start a new
+	   * search? How do we know which case-importance they wanted?
+	   */
+	  case 'n':
+	  case 'N':
+		/* highlight NEXT match */
+  		if (strlen(look_for)>1)
+		{
+		  hit = find_next(us, hit, look_for, case_matters);
+
+		  if (hit == -1) { wbell(); wflush(); hit=0; break; }
+		  drawhist_look(b_us, hit, 1, look_for, case_matters);
+		}
+		else	/* no search pattern... */
+		  {
+		    wbell();
+		    werror(_("No previous search!\n  Please 's' or 'S' first!"));
+		}
+		wflush();
+		break;
+#endif /*_SEARCH_HISTORY*/
+
 	  case 'u':
+	  case 'U':
 	  case K_UP:
 		if (y <= 0) break;
 		y--;
 		wscroll(b_us, S_DOWN);
-		wdrawelm(b_us, 0, getline(b_us, y));
+
+#if _SEARCH_HISTORY
+		/*
+		 * fmg 8/20/97
+		 * This is needed so that the movement in window will HIGHLIGHT
+		 * the lines that have the pattern we wanted... it's just nice.
+		 * This almost beggs for a function :-)
+		 */
+        	tmp_e = getline(b_us, y);
+		if (strlen(look_for) > 1)
+		{
+		  /* quick scan for pattern match */
+		  wdrawelm_var(b_us, 0, tmp_e, tmp_line);
+		  if (strlen(tmp_line)>1) {
+		    if (StrStr(tmp_line, look_for, case_matters))
+		      wdrawelm_inverse(b_us, 0, tmp_e);
+		    else
+		      wdrawelm(b_us, 0, tmp_e);
+		  }
+		}
+        	else
+                	wdrawelm(b_us, 0, tmp_e);
+#else
+                wdrawelm(b_us, 0, getline(b_us, y));
+#endif /*_SEARCH_HISTORY*/
 		wflush();
 		break;
 	  case 'd':
+	  case 'D':
 	  case K_DN:
 		if (y >= us->histlines) break;
 		y++;
 		wscroll(b_us, S_UP);
+
+#if _SEARCH_HISTORY
+		/*
+		 * fmg 8/20/97
+		 * This is needed so that the movement in window will HIGHLIGHT
+		 * the lines that have the pattern we wanted... it's just nice.
+		 * This almost beggs for a function :-)
+		 */
+        	tmp_e = getline(b_us, y + b_us->ys - 1);
+		if (strlen(look_for) > 1)
+		{
+			/* quick scan for pattern match */
+        		wdrawelm_var(b_us, 0, tmp_e, tmp_line);
+        		if (strlen(tmp_line)>1) {
+			  if (StrStr(tmp_line, look_for, case_matters))
+			    wdrawelm_inverse(b_us, b_us->ys - 1, tmp_e);
+			  else
+			    wdrawelm(b_us, b_us->ys - 1, tmp_e);
+			}
+		}
+        	else
+			wdrawelm(b_us, b_us->ys - 1, tmp_e);
+#else
 		wdrawelm(b_us, b_us->ys - 1, getline(b_us, y + b_us->ys - 1));
+#endif /*_SEARCH_HISTORY*/
 		wflush();
 		break;
 	  case 'b':
+	  case 'B':
 	  case K_PGUP:
 		if (y <= 0) break;
 		y -= b_us->ys;
 		if (y < 0) y = 0;
-		drawhist(b_us, y, 1);
+
+#if _SEARCH_HISTORY
+		/*
+		 * fmg 8/20/97
+		 * This is needed so that the movement in window will HIGHLIGHT
+		 * the lines that have the pattern we wanted... it's just nice.
+		 * Highlight any matches
+		 */
+  		if (strlen(look_for)>1 && us->histline)
+			drawhist_look(b_us, y, 1, look_for, case_matters);
+  		else
+			drawhist(b_us, y, 1);
+#else
+			drawhist(b_us, y, 1);
+#endif /*_SEARCH_HISTORY*/
 		break;
 	  case 'f':
+	  case 'F':
+	  case ' ': /* filipg: space bar will go page-down... pager-like */
 	  case K_PGDN:
 		if (y >= us->histlines) break;
 		y += b_us->ys;
-		if (y > us->histlines) y = us->histlines;
+		if (y > us->histlines)
+		{ y=us->histlines; wbell(); break; }
+
+#if _SEARCH_HISTORY
+		/*
+		 * fmg 8/20/97
+		 * This is needed so that the movement in window will HIGHLIGHT
+		 * the lines that have the pattern we wanted... it's just nice.
+		 * Highlight any matches
+		 */
+  		if (strlen(look_for)>1 && us->histline)
+			drawhist_look(b_us, y, 1, look_for, case_matters);
+  		else
+			drawhist(b_us, y, 1);
+#else
 		drawhist(b_us, y, 1);
+#endif /*_SEARCH_HISTORY*/
 		break;
 	}
   }
@@ -243,8 +600,9 @@ static void scrollback()
   wclose(b_st, 1);
   wlocate(us, us->curx, us->cury);
   wflush();
+  wredraw(us, 1);
 }
-#endif
+#endif /*HOSTORY*/
 
 #ifdef SIGWINCH
 /* The window size has changed. Re-initialize. */
@@ -255,7 +613,7 @@ int sig;
   size_changed = 1;
   signal(SIGWINCH, change_size);
 }
-#endif
+#endif /*SIGWINCH*/
 
 /*
  * Read a word from strings 's' and advance pointer.
@@ -282,16 +640,16 @@ char **s;
 }
 
 static char *bletch = 
-  "Usage: minicom [-somlz] [-c on] [-a on] [-t TERM] [-d entry] [-p ttyp] [configuration]\n";
+  N_("Usage: minicom [-somlz] [-c on] [-a on] [-t TERM] [-d entry] [-p ttyp] [configuration]\n");
 
 static void usage(env_args, optind, mc)
 int env_args, optind;
 char *mc;
 {
   if (env_args >= optind && mc)
-  	fprintf(stderr, "Wrong option in environment MINICOM=%s\n", mc);
-  fprintf(stderr, bletch);
-  fprintf(stderr, "Type \"minicom -h\" for help.\n");
+  	fprintf(stderr, _("Wrong option in environment MINICOM=%s\n"), mc);
+  fprintf(stderr, _(bletch));
+  fprintf(stderr, _("Type \"minicom -h\" for help.\n"));
   exit(1);
 }
 
@@ -302,26 +660,26 @@ static void helpthem()
 
   printf("\n%s", Version);
 #ifdef __DATE__
-  printf(" (compiled %s)", __DATE__);
-#endif
-  printf("(c) Miquel van Smoorenburg\n\n%s\n", bletch);
-  printf("  -s             : enter setup mode (only as root)\n");
-  printf("  -o             : do not initialize modem & lockfiles at startup\n");
-  printf("  -m             : use meta or alt key for commands\n");
-  printf("  -l             : literal ; assume screen uses the IBM-PC character set\n");
-  /* printf("  -L             : Ditto, but assume screen uses ISO8859\n"); */
-  printf("  -z             : try to use terminal's status line\n");
-  printf("  -c [on | off]  : ANSI style color usage on or off\n");
-  printf("  -a [on | off]  : use reverse or highlight attributes on or off\n");
-  printf("  -t term        : override TERM environment variable\n");
-  printf("  -d entry       : dial `entry' from the dialing directory\n");
-  printf("  -p ttyp..      : connect to pseudo terminal\n");
-  printf("  configuration  : configuration file to use\n\n");
-  printf("These options can also be specified in the MINICOM environment variable.\n");
-  printf("This variable is currently %s%s.\n", mc ? "set to " : "unset",
+  printf(_(" (compiled %s)"), __DATE__);
+#endif /*__DATE_*/
+  printf(_("(c) Miquel van Smoorenburg\n\n%s\n"), _(bletch));
+  printf(_("  -s             : enter setup mode (only as root)\n"));
+  printf(_("  -o             : do not initialize modem & lockfiles at startup\n"));
+  printf(_("  -m             : use meta or alt key for commands\n"));
+  printf(_("  -l             : literal ; assume screen uses the IBM-PC character set\n"));
+  printf(_("  -L             : Ditto, but assume screen uses ISO8859\n")); 
+  printf(_("  -z             : try to use terminal's status line\n"));
+  printf(_("  -c [on | off]  : ANSI style color usage on or off\n"));
+  printf(_("  -a [on | off]  : use reverse or highlight attributes on or off\n"));
+  printf(_("  -t term        : override TERM environment variable\n"));
+  printf(_("  -d entry       : dial `entry' from the dialing directory\n"));
+  printf(_("  -p ttyp..      : connect to pseudo terminal\n"));
+  printf(_("  configuration  : configuration file to use\n\n"));
+  printf(_("These options can also be specified in the MINICOM environment variable.\n"));
+  printf(_("This variable is currently %s%s.\n"), mc ? _("set to ") : _("unset"),
 	mc ? mc : "");
-  printf("The LIBDIR to find the configuration files and the\n");
-  printf("access file minicom.users is compiled as %s.\n\n", LIBDIR);
+  printf(_("The LIBDIR to find the configuration files and the\n"));
+  printf(_("access file minicom.users is compiled as %s.\n\n"), LIBDIR);
 
 #if 0  /* More than 24 lines.. */
   printf("\
@@ -356,8 +714,14 @@ char **argv;
   char *mc;			/* For 'MINICOM' env. variable */
   int env_args;			/* Number of args in env. variable */
   char *cmd_dial;		/* Entry from the command line. */
-  int alt_code;			/* Type of alt key */
+  int alt_code = 0;		/* Type of alt key */
   char pseudo[64];
+/*  char* console_encoding = getenv ("LC_CTYPE"); */
+
+#ifdef _I18N_
+  bindtextdomain("minicom", "/usr/share/locale");
+  textdomain("minicom");
+#endif
 
   /* Initialize global variables */
   portfd = -1;
@@ -369,7 +733,7 @@ char **argv;
   addlf = 0;
 #ifdef _SELECT
   local_echo = 0;
-#endif
+#endif /*SELECT*/
   strcpy(capname, "minicom.cap");
   lockfile[0] = 0;
   tempst = 0;
@@ -380,7 +744,7 @@ char **argv;
   screen_ibmpc = 0;
   screen_iso = 0;
   useattr = 1;
-  strcpy(termtype, getenv("TERM") ? getenv("TERM") : "dumb");
+  strncpy(termtype, getenv("TERM") ? getenv("TERM") : "dumb", sizeof(termtype));
   stdattr = XA_NORMAL;
   use_port = "dfl";
   alt_override = 0;
@@ -407,6 +771,11 @@ char **argv;
   sbcolor = RED;
   st_attr = XA_NORMAL;
   
+/* acme@conectiva.com.br 28/02/1998 */
+  /*  if (console_encoding != NULL)
+    if (strncmp (console_encoding, "ISO-8859", 8) == 0)
+    screen_iso++; */
+
 /* MARK updated 02/17/95 default history buffer size */
   num_hist_lines = 256;
 
@@ -436,10 +805,10 @@ char **argv;
 
   do {
 	/* Process options with getopt */
-	while((c = getopt(argk, args, "zhlsomMbc:a:t:d:p:")) != EOF) switch(c) {
+	while((c = getopt(argk, args, "zhlLsomMbc:a:t:d:p:")) != EOF) switch(c) {
   		case 's': /* setup */
   			if (real_uid != 0 && real_uid != eff_uid) {
-		fprintf(stderr, "minicom: -s switch needs root privilige\n");
+		fprintf(stderr, _("minicom: -s switch needs root privilige\n"));
 				exit(1);
 			}
   			dosetup = 1;
@@ -453,10 +822,10 @@ char **argv;
 				optarg += 5;
 			if (strncmp(optarg, "tty", 3) != 0 ||
 			    !strchr("pqrstuvwxyz", optarg[3])) {
-				fprintf(stderr, "minicom: argument to -p must be a pty\n");
+				fprintf(stderr, _("minicom: argument to -p must be a pty\n"));
 				exit(1);
 			}
-			sprintf(pseudo, "/dev/%s", optarg);
+			snprintf(pseudo, sizeof(pseudo), "/dev/%s", optarg);
 			dial_tty = pseudo;
 
 			/* Drop priviliges. */
@@ -477,7 +846,7 @@ char **argv;
 			screen_iso++;
 			break;
 		case 't': /* Terminal type */
-			strcpy(termtype, optarg);
+			strncpy(termtype, optarg, sizeof(termtype));
 #ifdef __linux__
 			/* Bug in older libc's (< 4.5.26 I think) */
 			if ((s = getenv("TERMCAP")) != NULL && *s != '/')
@@ -529,37 +898,39 @@ char **argv;
     /* Loop again if more options */
   } while(optind < argk);
 
-  if (real_uid == 0 && dosetup == 0) {
-	fprintf(stderr, "%s%s%s",
-  "minicom: WARNING: please don't run minicom as root when not maintaining\n",
-  "                  it (with the -s switch) since all changes to the\n",
-  "                  configuration will be GLOBAL !.\n");
-	sleep(5);
-  }
+/*
+ * if (real_uid == 0 && dosetup == 0) {
+ *	fprintf(stderr, "%s%s%s",
+ *  "minicom: WARNING: please don't run minicom as root when not maintaining\n",
+ *  "                  it (with the -s switch) since all changes to the\n",
+ *  "                  configuration will be GLOBAL !.\n");
+ *	sleep(5);
+ * }
+*/
 
   /* Avoid fraude ! */	
   for(s = use_port; *s; s++) if (*s == '/') *s = '_';
-  sprintf(parfile, "%s/minirc.%s", LIBDIR, use_port);
+  snprintf(parfile, sizeof(parfile), "%s/minirc.%s", LIBDIR, use_port);
 
   /* Get password file information of this user. */
   if ((pwd = getpwuid(real_uid)) == (struct passwd *)0) {
-  	fprintf(stderr, "You don't exist. Go away.\n");
+  	fprintf(stderr, _("You don't exist. Go away.\n"));
   	exit(1);
   }
 
   /* Remember home directory and username. */
   if ((s = getenv("HOME")) == CNULL)
-	strcpy(homedir, pwd->pw_dir);
+	strncpy(homedir, pwd->pw_dir, sizeof(homedir));
   else
-	strcpy(homedir, s);
-  strcpy(username, pwd->pw_name);
+	strncpy(homedir, s, sizeof(homedir));
+  strncpy(username, pwd->pw_name, sizeof(username));
 
   /* Get personal parameter file */
-  sprintf(pparfile, "%s/.minirc.%s", homedir, use_port);
+  snprintf(pparfile, sizeof(pparfile), "%s/.minirc.%s", homedir, use_port);
 
   /* Check this user in the USERFILE */
   if (real_uid != 0 && real_uid != eff_uid) {
-  	sprintf(userfile, "%s/minicom.users", LIBDIR);
+  	snprintf(userfile, sizeof(userfile), "%s/minicom.users", LIBDIR);
 	if ((fp = fopen(userfile, "r")) != (FILE *)0) {
 		while(fgets(buf, 70, fp) != CNULL && !userok) {
 			/* Read first word */
@@ -581,9 +952,9 @@ char **argv;
 		fclose(fp);
 		if (!userok) {
 			fprintf(stderr,
-   "Sorry %s. You are not allowed to use configuration %s.\n",
+   _("Sorry %s. You are not allowed to use configuration %s.\n"),
 				pwd->pw_name, use_port);
-			fprintf(stderr, "Ask your sysadm to add your name to %s\n",
+			fprintf(stderr, _("Ask your sysadm to add your name to %s\n"),
 				userfile);
 			exit(1);
 		}
@@ -592,7 +963,6 @@ char **argv;
   buf[0] = 0;
 
   read_parms();
-  if (dial_tty == NULL) dial_tty = P_PORT;
 
   stdwin = NIL_WIN; /* It better be! */
 
@@ -602,7 +972,18 @@ char **argv;
   	mbcolor = tbcolor = sbcolor = BLACK;
 	st_attr = XA_REVERSE;
   }
-  if (!dosetup && open_term(doinit) < 0) exit(1);
+
+  if (dial_tty == NULL) {
+    if (!dosetup) {
+      while((dial_tty = get_port(P_PORT)) != NULL && open_term(doinit) < 0)
+	;
+      if(dial_tty == NULL)
+	exit(1);
+    }
+  }
+  else {
+    if (!dosetup && open_term(doinit) < 0) exit(1);
+  }
 
   mc_setenv("TERM", termtype);
 
@@ -617,7 +998,10 @@ char **argv;
   		wclose(stdwin, 1);
   		exit(0);
   	}
-  	if (open_term(doinit) < 0) exit(1);
+	while((dial_tty = get_port(P_PORT)) != NULL && open_term(doinit) < 0)
+	  ;
+	if(dial_tty == NULL)
+	  exit(1);
   }
 
   /* Signal handling */
@@ -688,8 +1072,13 @@ char **argv;
  
   if (doinit) modeminit();
  
-  wputs(us, CR_VERSION);
-  wprintf(us, "\nPress %sZ for help on special keys\r\n\n", esc_key());
+  wprintf(us, "\n%s\r\n", _(CR_VERSION));
+  wprintf(us, "\n%s: %s%s%s%s%s\r\n", _("OPTIONS"),
+	  CR_OPTION1,CR_OPTION2,CR_OPTION3,CR_OPTION4,CR_OPTION5);
+#if defined (__DATE__) && defined (__TIME__)
+  wprintf(us, "%s %s, %s.\r\n",_("Compiled on"), __DATE__,__TIME__);
+#endif
+  wprintf(us, _("\nPress %sZ for help on special keys\n\n"), esc_key());
 
   /* Now that all initialization is done, drop our priviliges. */
   drop_privs();
@@ -706,14 +1095,15 @@ dirty_goto:
 		case 'a': /* Add line feed */
 			addlf = !addlf;
 			vt_set(addlf, -1, NULL, -1, -1, -1, -1);
-			s = addlf ? "Add linefeed ON" : "Add linefeed OFF";
+			s = addlf ? _("Add linefeed ON") : _("Add linefeed OFF");
 			werror(s);
 			break;
 #if _SELECT
 		case 'e': /* Local echo on/off. */
 			local_echo = !local_echo;
 			vt_set(-1, -1, NULL, -1, -1, local_echo, -1);
-			s = local_echo ? "Local echo ON" : "Local echo OFF";
+			s = local_echo ?
+			  _("Local echo ON") : _("Local echo OFF");
 			werror(s);
 			break;
 #endif
@@ -727,7 +1117,7 @@ dirty_goto:
 		case 'f': /* Send break */
 			sendbreak();
 			break;
-#if HISTORY
+#if _HISTORY
 		case 'b': /* Scroll back */
 			scrollback();
 			break;
@@ -736,11 +1126,11 @@ dirty_goto:
 			modeminit();
 			break;	
 		case 'q': /* Exit without resetting */
-			c = ask("Leave without reset?", c1);
+			c = ask(_("Leave without reset?"), c1);
 			if (c == 0) quit = NORESET;
 #if _HAVE_MACROS
 			if (!strcmp(P_MACCHG,"CHANGED")) {
-				c = ask ("Save macros?",c1);
+				c = ask (_("Save macros?"),c1);
 				if (c == 0)
 					if (dodflsave() < 0) /* fmg - error */
 					{
@@ -752,7 +1142,7 @@ dirty_goto:
 #endif
 			break;
 		case 'x': /* Exit Minicom */
-			c = ask("Leave Minicom?", c1);
+			c = ask(_("Leave Minicom?"), c1);
 			if (c == 0) {
 				quit = RESET;
 				if(online >= 0)
@@ -761,7 +1151,7 @@ dirty_goto:
 			}
 #if _HAVE_MACROS
 			if (!strcmp(P_MACCHG,"CHANGED")) {
-				c = ask ("Save macros?",c1);
+				c = ask (_("Save macros?"),c1);
 				if (c == 0)
 					if (dodflsave() < 0) /* fmg - error */
 					{
@@ -774,15 +1164,16 @@ dirty_goto:
 			break;
 		case 'l': /* Capture file */
 			if (capfp == (FILE *)0 && !docap) {
-				s = input("Capture to which file? ", capname);
+				s = input(_("Capture to which file? "),
+					  capname);
 				if (s == CNULL || *s == 0) break;
 				if ((capfp = sfopen(s, "a")) == (FILE *)NULL) {
-					werror("Cannot open capture file");
+					werror(_("Cannot open capture file"));
 					break;
 				}
 				docap = 1;
 			} else if (capfp != (FILE *)0 && !docap) {
-				c = ask("Capture file", c3);
+				c = ask(_("Capture file"), c3);
 				if (c == 0) {
 					fclose(capfp);
 					capfp = (FILE *)NULL;
@@ -790,7 +1181,7 @@ dirty_goto:
 				}
 				if (c == 1) docap = 1;
 			} else if (capfp != (FILE *)0 && docap) {
-				c = ask("Capture file", c2);
+				c = ask(_("Capture file"), c2);
 				if (c == 0) {
 					fclose(capfp);
 					capfp = (FILE *)NULL;
@@ -822,7 +1213,7 @@ dirty_goto:
 		case 'w': /* Line wrap on-off */	
 			c = (!us->wrap);
 			vt_set(addlf, c, capfp, docap, -1, -1, -1);
-			s = c ? "Linewrap ON" : "Linewrap OFF";
+			s = c ? _("Linewrap ON") : _("Linewrap OFF");
 			werror(s);
 			break;
 		case 'o': /* Configure Minicom */
@@ -853,8 +1244,11 @@ dirty_goto:
   /* Reset parameters */
   if (quit != NORESET)
 	m_restorestate(portfd);
-  else
+  else {
+        if (online > 0)
+	  do_log(_("Quit without reset while online."));
 	m_hupcl(portfd, 0);
+  }
   if (capfp != (FILE *)0) fclose(capfp);
   wclose(us, 0);
   wclose(st, 0);
