@@ -10,7 +10,13 @@
  *		2 of the License, or (at your option) any later version.
  *
  * fmg 1/11/94 colors
- *
+ * fmg 8/22/97 History Buffer Search support
+ * jl  23.06.97 sec parameter to m_dtrtoggle
+ * jl  04.09.97 conversion tables
+ * jl  09.09.97 loadconv and saveconv protos
+ * jl  05.10.97 changed return value of dial() to long
+ * jseymour@jimsun.LinxNet.com (Jim Seymour) 03/26/98 - Added prototype
+ *    for new "get_port()" function in util.c.
  */
 
 /* First include all other application-dependant include files. */
@@ -71,6 +77,14 @@ EXTERN int portuid, portgid; /* Uid and Gid of /dev/ttyS... */
 EXTERN char *dial_name;	    /* System we're conneced to */
 EXTERN char *dial_number;   /* Number we've dialed. */
 
+/*
+ * fmg 8/22/97
+ * Search pattern can be THIS long (x characters)
+ */
+#if _SEARCH_HISTORY
+#define MAX_SEARCH      30
+#endif /*SEARCH_HISTORY*/
+
 /* fmg 1/11/94 colors */
 
 EXTERN int mfcolor;     /* Menu Foreground Color */
@@ -81,6 +95,9 @@ EXTERN int sfcolor;     /* Status Bar Foreground Color */
 EXTERN int sbcolor;     /* Status Bar Background Color */
 EXTERN int st_attr;	/* Status Bar attributes. */
 
+/* jl 04.09.97 conversion tables */
+EXTERN unsigned char vt_outmap[256], vt_inmap[256];
+
 /* MARK updated 02/17/95 - history buffer */
 EXTERN int num_hist_lines;  /* History buffer size */
 
@@ -111,6 +128,8 @@ _PROTO(int dotermmenu, ( void ));
 _PROTO(int dodflsave, ( void ));	/* fmg - need it */
 _PROTO(void vdodflsave, ( void ));	/* fmg - need it */
 _PROTO(int domacsave, ( void ));	/* fmg - need it */
+_PROTO(int loadconv, ( char *));	/* jl */
+_PROTO(int saveconv, ( char *));	/* jl */
 
 /*      Prototypes from file: " dial.c "      */
 
@@ -119,15 +138,20 @@ _PROTO(void modeminit, ( void ));
 _PROTO(void modemreset, ( void ));
 _PROTO(void hangup, ( void ));
 _PROTO(void sendbreak, ( void ));
-_PROTO(int dial, ( struct dialent *d ));
+_PROTO(long dial, ( struct dialent *d , struct dialent **d2 ));
 _PROTO(int readdialdir, ( void ));
 _PROTO(void dialone, ( char *entry ));
 _PROTO(void dialdir, ( void ));
 
-/*      Prototypes from file: " fastsystem.c "      */
+/*      Prototypes from file: " file.c "      */
+
+_PROTO(char *filedir, (int how_many, int downloading));
+
+/*      Prototypes from file: " util.c "      */
 
 _PROTO(int fastexec, ( char *cmd ));
 _PROTO(int fastsystem, ( char *cmd , char *in , char *out , char *err ));
+_PROTO(char * get_port, ( char * ));
 
 /*      Prototypes from file: " help.c "      */
 
@@ -158,11 +182,24 @@ _PROTO(void curs_status, ( void ));
 _PROTO(void show_status, ( void ));
 _PROTO(void scriptname, ( char *s ));
 _PROTO(int do_terminal, ( void ));
+_PROTO(void do_log, ( char *line ));
 
 /*      Prototypes from file: " minicom.c "      */
 
 _PROTO(void port_init, ( ));
 
+/*
+ * fmg 8/22/97
+ * Prototypes needed...
+ */
+#if _SEARCH_HISTORY
+_PROTO(void drawhist_look, (WIN *w, int y, int r, char *look, int case_matters));
+_PROTO(void searchhist, (WIN *w_orig, WIN *w_hist, int r, char *str));
+_PROTO(int find_next, (WIN *w, int hit_line, char *look, int case_matters));
+_PROTO(const char *upcase, (char *dest, char *src));
+_PROTO(char *StrStr, (char *str1, char *str2, int case_matters));
+#endif /*SEARCH_HISTORY*/
+
 /*      Prototypes from file: " rwconf.c "      */
 
 _PROTO(int writepars, ( FILE *fp , int all ));
@@ -180,7 +217,7 @@ _PROTO(int dup2, ( int from, int to ));
 /*      Prototypes from file: " sysdep1.c "      */
 
 _PROTO(void m_sethwf, ( int fd , int on ));
-_PROTO(void m_dtrtoggle, ( int fd ));
+_PROTO(void m_dtrtoggle, ( int fd , int sec ));
 _PROTO(void m_break, ( int fd ));
 _PROTO(int m_getdcd, ( int fd ));
 _PROTO(void m_setdcd, ( int fd , int what ));
