@@ -15,10 +15,16 @@
  *                  works with these patches. TODO: make these alloc
  *                  memory dynamically... it's nice to have a 15K macro
  *                  _WHEN_ it's being (not like now with -DMAC_LEN) :-)
- *
+ * // jl  23.06.97 - changed mdropdtr to numeric
+ * // jl  04.09.97 - conversion table filename added to mpars table
+ * // jl  22.02.98 - file selection window setting added to mpars table
+ * // acme 26.02.98 - i18n
+ * // acme 18.03.98 - more i18n
+ * // jl  05.04.98 - added the multifile parameter for transfer protocols
  */
 #include "port.h"
 #include "minicom.h"
+#include "intl.h"
 
 #define ADM_CHANGE	1
 #define USR_CHANGE	2
@@ -47,25 +53,25 @@ struct macs mmacs[] = {
 struct pars mpars[] = {
   /* Protocols */
   /* Warning: minicom assumes the first 12 entries are these proto's ! */
-  { "YUNYzmodem",	PUBLIC,   "pname1" },
-  { "YUNYymodem",	PUBLIC,   "pname2" },
-  { "YUNYxmodem",	PUBLIC,   "pname3" },
-  { "NDNYzmodem",	PUBLIC,   "pname4" },
-  { "NDNYymodem",	PUBLIC,   "pname5" },
-  { "YDNYxmodem",	PUBLIC,   "pname6" },
-  { "YUYNkermit",	PUBLIC,   "pname7" },
-  { "NDYNkermit",	PUBLIC,   "pname8" },
-  { "",			PUBLIC,   "pname9" },
+  { "YUNYYzmodem",	PUBLIC,   "pname1" },
+  { "YUNYYymodem",	PUBLIC,   "pname2" },
+  { "YUNYNxmodem",	PUBLIC,   "pname3" },
+  { "NDNYYzmodem",	PUBLIC,   "pname4" },
+  { "NDNYYymodem",	PUBLIC,   "pname5" },
+  { "YDNYNxmodem",	PUBLIC,   "pname6" },
+  { "YUYNNkermit",	PUBLIC,   "pname7" },
+  { "NDYNNkermit",	PUBLIC,   "pname8" },
+  { "YUNYNascii",	PUBLIC,   "pname9" },
   { "",			PUBLIC,   "pname10" },
   { "",			PUBLIC,   "pname11" },
   { "",			PUBLIC,   "pname12" },
 #ifdef __linux__
-  { "/usr/bin/sz -vv",	PUBLIC,   "pprog1" },
-  { "/usr/bin/sb -vv",	PUBLIC,   "pprog2" },
-  { "/usr/bin/sx -vv",	PUBLIC,   "pprog3" },
-  { "/usr/bin/rz -vv",	PUBLIC,   "pprog4" },
-  { "/usr/bin/rb -vv",	PUBLIC,   "pprog5" },
-  { "/usr/bin/rx -vv",	PUBLIC,   "pprog6" },
+  { "/usr/bin/sz -vv -b",	PUBLIC,   "pprog1" },
+  { "/usr/bin/sb -vv",		PUBLIC,   "pprog2" },
+  { "/usr/bin/sx -vv",		PUBLIC,   "pprog3" },
+  { "/usr/bin/rz -vv -b -E",	PUBLIC,   "pprog4" },
+  { "/usr/bin/rb -vv",		PUBLIC,   "pprog5" },
+  { "/usr/bin/rx -vv",		PUBLIC,   "pprog6" },
   { "/usr/bin/kermit -i -l %l -s", PUBLIC, "pprog7" },
   { "/usr/bin/kermit -i -l %l -r", PUBLIC, "pprog8" },
 #else
@@ -79,7 +85,7 @@ struct pars mpars[] = {
   { "/usr/local/bin/kermit -i -l %l -s", PUBLIC, "pprog7" },
   { "/usr/local/bin/kermit -i -l %l -r", PUBLIC, "pprog8" },
 #endif
-  { "",			PUBLIC,   "pprog9" },
+  { "/usr/bin/ascii-xfr -dsv", PUBLIC,   "pprog9" },
   { "",			PUBLIC,   "pprog10" },
   { "",			PUBLIC,   "pprog11" },
   { "",			PUBLIC,   "pprog12" },
@@ -93,8 +99,8 @@ struct pars mpars[] = {
   { "N",		PUBLIC,   "parity" },
   /* Kermit the frog */
   { KERMIT,		PRIVATE,  "kermit" },
-  { "Yes",		PRIVATE,  "kermallow" },
-  { "No",		PRIVATE,  "kermreal" },
+  { N_("Yes"),		PRIVATE,  "kermallow" },
+  { N_("No"),		PRIVATE,  "kermreal" },
   { "3",		PUBLIC,   "colusage" },
   /* The script program */
   { "runscript",	PUBLIC,   "scriptprog" },
@@ -115,19 +121,19 @@ struct pars mpars[] = {
   { "~~+++~~ATH^M",	PUBLIC,   "mhangup" },
   { "^M",		PUBLIC,   "mdialcan" },
   { "45",		PUBLIC,   "mdialtime" },
-  { "5",		PUBLIC,   "mrdelay" },
+  { "2",		PUBLIC,   "mrdelay" },
   { "10",		PUBLIC,   "mretries" },
   { "No",		PUBLIC,   "mautobaud" },
-  { "Yes",		PUBLIC,   "mdropdtr" },
+  { "1",		PUBLIC,   "mdropdtr" },   /* jl 23.06.97 */
   { "",			PUBLIC,   "updir" },
   { "",			PUBLIC,   "downdir" },
   { "",			PUBLIC,   "scriptdir" },
-  { "^A",		PUBLIC,   "escape-key" },
-  { "DEL",		PUBLIC,   "backspace" },
-  { "enabled",		PUBLIC,   "statusline" },
-  { "Yes",		PUBLIC,   "hasdcd" },
-  { "Yes",		PUBLIC,   "rtscts" },
-  { "No",		PUBLIC,   "xonxoff" },
+  { "Escape (Meta)",	PUBLIC,   "escape-key" },
+  { "BS",		PUBLIC,   "backspace" },
+  { N_("enabled"),	PUBLIC,   "statusline" },
+  { N_("Yes"),		PUBLIC,   "hasdcd" },
+  { N_("Yes"),		PUBLIC,   "rtscts" },
+  { N_("No"),		PUBLIC,   "xonxoff" },
   { "D",		PUBLIC,   "zauto" },
 
   /* fmg 1/11/94 colors */
@@ -148,9 +154,15 @@ struct pars mpars[] = {
 
   /* Continue here with new stuff. */
   { "Yes",		PUBLIC,   "sound"  },
-
   /* MARK updated 02/17/95 - History buffer size */
-  { "256",             PUBLIC,    "histlines" },
+  { "2000",             PUBLIC,   "histlines" },
+
+  /* Character conversion table - jl / 04.09.97 */
+  { "",                	PUBLIC,    "convf" },
+  /* Do you want to use the filename selection window? */
+  { "Yes",		PUBLIC,	   "fselw" },
+  /* Do you want to be prompted for the download directory? */
+  { "No",		PUBLIC,    "askdndir" },
 
   /* That's all folks */
   { "",                 0,         (char *)NULL },
@@ -185,9 +197,9 @@ int all;
   struct pars *p;
 
   if (all)
-	fprintf(fp, "# Machine-generated file - use \"minicom -s\" to change parameters.\n");
+	fprintf(fp, _("# Machine-generated file - use \"minicom -s\" to change parameters.\n"));
   else
-	fprintf(fp, "# Machine-generated file - use setup menu in minicom to change parameters.\n");
+	fprintf(fp, _("# Machine-generated file - use setup menu in minicom to change parameters.\n"));
 
   for(p = mpars; p->desc; p++)
   	if ((all && (p->flags & ADM_CHANGE)) ||
@@ -239,17 +251,17 @@ int init;
 			if (((p->flags & PRIVATE) && public) ||
 			    ((p->flags & PUBLIC)  && !public)) {
 				fprintf(stderr,
-			"** Parameter %s is %s, but is marked %s in %s config file\n",
+			_("** Parameter %s is %s, but is marked %s in %s config file\n"),
 					s,
 					(p->flags & PRIVATE) ? "private" : "public",
 					public ? "public" : "private",
-					init ? "global" : "personal");
+					init ? _("global") : _("personal"));
 				dosleep = 1;
 			}
 			/* See if user is allowed to change this flag. */
 			if ((p->flags & PRIVATE) && public == 1) {
 				fprintf(stderr,
-			"== Attempt to change private parameter %s - denied.\n",
+		       _("== Attempt to change private parameter %s - denied.\n"),
 					p->desc);
 				dosleep = 1;
 				continue;
@@ -260,6 +272,16 @@ int init;
   				s = "";
   			while(*s && (*s == '\t' || *s == ' ')) s++;
 
+			/* We must be prepared for config files saved before
+			   the P_MUL parameter was added. So, we make room
+			   for P_MUL if necessary - jl 5.4.98 */
+			if (!strncmp(p->desc,"pname",(size_t) 5)) {
+			  if (s[4] != 'N' && s[4] != 'Y') {
+			    memmove (s+5, s+4, strlen(s+4));
+			    *(s+4)='N';
+			  }
+			}
+
   			/* If the same as default, don't mark as changed */
   			if (strcmp(p->value, s) == 0) {
   				p->flags = 0;
