@@ -614,7 +614,7 @@ char *text;
 
   w = getword(&text);
   if (w == CNULL || *text) syntaxerr("(in goto/gosub label)");
-  sprintf(buf, "%s:", w);
+  snprintf(buf, sizeof(buf), "%s:", w);
   len = strlen(buf);
   for(l = curenv->lines; l; l = l->next) if (!strncmp(l->line, buf, len)) break;
   if (l == LNULL) {
