@@ -22,9 +22,12 @@
  *		modify it under the terms of the GNU General Public License
  *		as published by the Free Software Foundation; either version
  *		2 of the License, or (at your option) any later version.
+ *
+ *   jl  23.06.97  adjustable DTR downtime
  */
 #include "sysdep.h"
 
+
 /* Set hardware flow control. */
 void m_sethwf(fd, on)
 int fd, on;
@@ -76,15 +79,18 @@ int fd;
 /*
  * Drop DTR line and raise it again.
  */
-void m_dtrtoggle(fd) 
+void m_dtrtoggle(fd,sec) 
 int fd;
+int sec;
 {
 #ifdef TIOCSDTR
 
   /* Use the ioctls meant for this type of thing. */
   ioctl(fd, TIOCCDTR, 0);
-  sleep(1);
-  ioctl(fd, TIOCSDTR, 0);
+  if (sec>0) {
+    sleep(sec);
+    ioctl(fd, TIOCSDTR, 0);
+  }
 
 #else /* TIOCSDTR */
 #  if defined (_POSIX) && !defined(_HPUX_SOURCE)
@@ -97,8 +103,10 @@ int fd;
   cfsetospeed(&tty, B0);
   cfsetispeed(&tty, B0);
   tcsetattr(fd, TCSANOW, &tty);
-  sleep(1);
-  tcsetattr(fd, TCSANOW, &old);
+  if (sec>0) {
+    sleep(sec);
+    tcsetattr(fd, TCSANOW, &old);
+  }
 
 #  else /* POSIX */
 #    ifdef _V7
@@ -111,8 +119,10 @@ int fd;
   
   ng.sg_ispeed = ng.sg_ospeed = 0;
   ioctl(fd, TIOCSETP, &ng);
-  sleep(1);
-  ioctl(fd, TIOCSETP, &sg);
+  if (sec>0) {
+    sleep(sec);
+    ioctl(fd, TIOCSETP, &sg);
+  }
 
 #    endif /* _V7 */
 #    ifdef _HPUX_SOURCE
@@ -121,8 +131,10 @@ int fd;
   ioctl(fd, MCSETAF, &mflag);
   ioctl(fd, MCGETA, &mflag);
   mflag = MRTS | MDTR;
-  sleep(1);
-  ioctl(fd, MCSETAF, &mflag);
+  if (sec>0) {
+    sleep(sec);
+    ioctl(fd, MCSETAF, &mflag);
+  }
 #    endif /* _HPUX_SOURCE */
 #  endif /* POSIX */
 #endif /* TIOCSDTR */
