@@ -9,10 +9,21 @@
  *		modify it under the terms of the GNU General Public License
  *		as published by the Free Software Foundation; either version
  *		2 of the License, or (at your option) any later version.
+ *
+ * jl 13.09.97	pass actual terminal lines (LINES - statusline)
+ *		to runscript in environment variable TERMLIN
+ * jl 16.09.97	logging of sz/rz file transfers
+ * jl 29.09.97	fix on the transfer logging
+ * hgk&jl 2.98	filename selection window
+ * acme 25.02.98 i18n
+ * js&jl 04.98	the better filename selection window
  */
 #include "port.h"
 #include "minicom.h"
+#include "intl.h"
 
+/*#define LOG_XFER	  debugging option to log all output of rz/sz
+ */
 static int udpid;
 
 /*
@@ -41,11 +52,13 @@ char *dir;
   }
   
   if(*dir != '/') {
-  	sprintf(buf, "%s/%s", homedir, dir);
+  	snprintf(buf, sizeof(buf), "%s/%s", homedir, dir);
   	dir = buf;
   }
   if (chdir(dir) < 0) {
-  	sprintf(err, "Cannot chdir to %.30s", dir);
+  	/* This may look safe but you might I8N change the string! so
+  	   snprintf it */
+  	snprintf(err, sizeof(err),  _("Cannot chdir to %.30s"), dir);
   	werror(err);
   	return(-1);
   }
@@ -81,11 +94,11 @@ char *s;
   	}
   	switch(*++s) {
   		case 'l':
-  			strcpy(buf + i, dial_tty);
+  			strncpy(buf + i, dial_tty, sizeof(buf)-i);
   			i += strlen(dial_tty) - 1;
   			break;
   		case 'b':
-  			strcpy(buf + i, P_BAUDRATE);
+  			strncpy(buf + i, P_BAUDRATE, sizeof(buf)-i);
   			i += strlen(P_BAUDRATE) - 1;
   			break;
   		default:
@@ -98,18 +111,61 @@ char *s;
   return(buf);
 }
 
+/*
+ * Trim the leading & trailing whitespaces from the string
+ * jl 15.09.97
+ */
+char *trim(char *outstring, char *instring, int n)
+{
+  char *p;
+  char *ip;
+  char *op;
+  char *np;
+
+  ip = instring;
+  np = ip + n;
+  while ((*ip <= ' ') && (ip < np)) ip++;
+
+  op = outstring;
+  np = op + n;
+  while ((*ip >= ' ') && (op <= np)) {
+    *op = *ip;
+    ip++;
+    op++;
+  }
+
+  if (op<np)
+    *op = 0;
+
+  while ((op > outstring) && (*op <= ' ')) {
+    *op = 0;
+    op--;
+  }
+
+  p = outstring;
+  return(p);
+}
+  
 /*
  * Choose from numerous up and download protocols!
  */
+
 void updown(what, nr)
 int what;
 int nr;
 {
+#ifdef LOG_XFER
+  #warning LOG_XFER defined!
+  FILE *xfl;
+#endif
   char *name[13];
   int idx[13];
   int r, f, g = 0;
-  char *t = what == 'U' ? "Upload" : "Download";
-  char buf[128];
+  char *t = what == 'U' ? _("Upload") : _("Download");
+  char buf[160];
+  char buffirst[20];
+  char xfrstr[160] = "";
+  char trimbuf[160] = "";
   char title[64];
   char *s ="";
   int pipefd[2];
@@ -142,25 +198,49 @@ int nr;
 
   buf[0] = 0;
 
-  if (P_PNN(g) == 'Y') {
-  	s = input("Please enter file names", buf);
-  	if (s == CNULL || *s == 0) return;
-  }
+/* jseymour file selector with choice of dir on zmodem, etc. download */
+#if 1
+  {
+	int multiple; /* 0:only directory, 1:one file, -1:any number */
+	
+	if(P_MUL(g)=='Y') {
+	    /* need file(s), or just a directory? */
+	    multiple = what == 'U'? -1 : 0;
+	}
+	else {
+	    multiple = 1;	/* only one allowed */
+	}
 
-  sprintf(cmdline, "%s %s", P_PPROG(g), s);
+	if (P_FSELW[0] == 'Y' && (what == 'U' || P_ASKDNDIR[0] == 'Y')) {
+	  s = filedir(multiple, what == 'U'? 0 : 1);
+	  if (s == NULL)
+	    return;
+	}
+	else if (P_PNN(g) == 'Y') {
+	  s = input(_("Please enter file names"), buf);
+	  if (s == NULL)
+	    return;
+	}
+
+	/* discard directory if "multiple" == 0 */
+	snprintf(cmdline, sizeof(cmdline), "%s %s", P_PPROG(g), multiple == 0? "" : s);
+  }
+#endif
+
+  do_log(cmdline);   /* jl 22.06.97 */
 
   if (P_PFULL(g) == 'N') {
-	win = wopen(10, 7, 70, 13, BSINGLE, stdattr, mfcolor, mbcolor, 1, 0, 1);
-	sprintf(title, "%.30s %s - Press CTRL-C to quit", P_PNAME(g),
-		what == 'U' ? "upload" : "download");
-	wtitle(win, TMID, title);
-	pipe(pipefd);
+    win = wopen(10, 7, 70, 13, BSINGLE, stdattr, mfcolor, mbcolor, 1, 0, 1);
+    snprintf(title, sizeof(title), _("%.30s %s - Press CTRL-C to quit"), P_PNAME(g),
+	    what == 'U' ? _("upload") : _("download"));
+    wtitle(win, TMID, title);
+    pipe(pipefd);
   } else
-	wleave();
+    wleave();
 
   switch(udpid = fork()) {
   	case -1:
-  		werror("Out of memory: could not fork()");
+  		werror(_("Out of memory: could not fork()"));
 		if (win) {
   			close(pipefd[0]);
   			close(pipefd[1]);
@@ -196,12 +276,47 @@ int nr;
   signal(SIGINT, udcatch);
   if (P_PIORED(g) == 'Y') {
 	close(pipefd[1]);
-	while((n = read(pipefd[0], buf, 80)) > 0) {
+#ifdef LOG_XFER
+	xfl=fopen("xfer.log","wb");
+#endif
+	while((n = read(pipefd[0], buf, sizeof(buf))) > 0) {
 	  	buf[n] = '\0';
   		wputs(win, buf);
 		timer_update();
+		/* Log the filenames & sizes 	jl 14.09.97 */
+#ifdef LOG_XFER
+		if (xfl)
+		  fprintf(xfl,">%s<\n",buf);
+#endif
+		if (sscanf(buf, "%19s", buffirst)) { /* if / jl 29.09.97 */
+		  if (!strncmp (buffirst, "Receiving", 9) ||
+		      !strncmp (buffirst, "Sending", 7)) {
+		    if (xfrstr[0]) {
+		      trim (trimbuf, xfrstr, sizeof(trimbuf));
+		      do_log (trimbuf);
+		      xfrstr[0] = 0;
+		    }
+		    trim (trimbuf, buf, sizeof(trimbuf));
+		    do_log(trimbuf);
+		  } else if (!strncmp (buffirst, "Bytes", 5)) {
+		    strncpy (xfrstr, buf, sizeof(xfrstr));
+		  }
+		  buffirst[0]=0;
+		  trimbuf[0]=0;
+		}
 	}
+#ifdef LOG_XFER
+	if (xfl)
+	  fclose(xfl);
+#endif
   }
+  /* Log the last file size	jl 14.09.97 */
+  if (xfrstr[0]) {
+    trim (trimbuf, xfrstr, sizeof(trimbuf));
+    do_log (trimbuf);
+    xfrstr[0] = 0;
+  }
+
   while( udpid != m_wait(&status) );
   if (win) {
 	enab_sig(0, 0);
@@ -216,14 +331,14 @@ int nr;
   m_flush(portfd);
   port_init();
   (void) setcbreak(2); /* Raw, no echo. */
-  close(pipefd[0]);
+  if (win) close(pipefd[0]);
   (void) mcd("");
 
   /* If we got interrupted, status != 0 */
   if (win && (status & 0xFF00) == 0) {
 #if VC_MUSIC
 	if (P_SOUND[0] == 'Y') {
-		wprintf(win, "\n READY: press any key to continue...");
+		wprintf(win, _("\n READY: press any key to continue..."));
 		music();
 	} else
 		sleep(1);
@@ -255,7 +370,7 @@ void kermit()
   switch(pid = fork()) {
   	case -1:
 		wreturn();
-  		werror("Out of memory: could not fork()");
+  		werror(_("Out of memory: could not fork()"));
   		return;
   	case 0: /* Child */
   		/* Remove lockfile */
@@ -283,10 +398,10 @@ void kermit()
 	n = umask(022);
   	/* Create lockfile compatible with UUCP-1.2 */
   	if ((fd = open(lockfile, O_WRONLY | O_CREAT | O_EXCL, 0666)) < 0) {
-  		werror("Cannot re-create lockfile!");
+  		werror(_("Cannot re-create lockfile!"));
   	} else {
 		(void) chown(lockfile, (uid_t)real_uid, (gid_t)real_gid);
-  		sprintf(buf, "%05d minicom %.20s\n", (int)getpid(), username);
+  		snprintf(buf, sizeof(buf),  "%05d minicom %.20s\n", (int)getpid(), username);
   		write(fd, buf, strlen(buf));
   		close(fd);
   	}
@@ -376,27 +491,31 @@ char *s, *l, *p;
   int n;
   int pipefd[2];
   char buf[81];
+  char scr_lines[5];
   char cmdline[128];
   char *ptr;
   WIN *w;
   int done = 0;
   char *msg = "Same as last";
+  char* username = _(" A -   Username        :"),
+      * password = _(" B -   Password        :"),
+      * name_of_script = _(" C -   Name of script  :"),
+      * question = _("Change which setting?     (Return to run, ESC to stop)");
+
 
   if (ask) {
 	w = wopen(10, 5, 70, 10, BDOUBLE, stdattr, mfcolor, mbcolor, 0, 0, 1);
-	wtitle(w, TMID, "Run a script");
+	wtitle(w, TMID, _("Run a script"));
 	wputs(w, "\n");
-	wprintf(w, " A -   Username        : %s\n",
-					scr_user[0] ? msg : "");
-	wprintf(w, " B -   Password        : %s\n",
-					scr_passwd[0] ? msg : "");
-	wprintf(w, " C -   Name of script  : %s\n", scr_name);
+	wprintf(w, "%s %s\n", username, scr_user[0] ? msg : "");
+	wprintf(w, "%s %s\n", password, scr_passwd[0] ? msg : "");
+	wprintf(w, "%s %s\n", name_of_script, scr_name);
 	wlocate(w, 4, 5);
-	wputs(w, "Change which setting?     (Return to run, ESC to stop)");
+	wputs(w, question);
 	wredraw(w, 1);
 
 	while(!done) {
-	    wlocate(w, 26, 5);
+	    wlocate(w, strlen (question) + 5, 5);
 	    n = wxgetch();
 	    if (islower(n)) n = toupper(n);
 	    switch(n) {
@@ -413,19 +532,19 @@ char *s, *l, *p;
 			wclose(w, 1);
 			return;
 		case 'A':
-			wlocate(w, 25, 1);
+			wlocate(w, strlen (username) + 1, 1);
 			wclreol(w);
 			scr_user[0] = 0;
 			wgets(w, scr_user, 32, 32);
 			break;
 		case 'B':
-			wlocate(w, 25, 2);
+			wlocate(w, strlen (password) + 1, 2);
 			wclreol(w);
 			scr_passwd[0] = 0;
 			wgets(w, scr_passwd, 32, 32);
 			break;
 		case 'C':
-			wlocate(w, 25, 3);
+			wlocate(w, strlen (name_of_script) + 1, 3);
 			wgets(w, scr_name, 32, 32);
 			break;
 		default:
@@ -433,10 +552,11 @@ char *s, *l, *p;
 	    }
 	}
   } else {
-  	strcpy(scr_user, l);
-  	strcpy(scr_name, s);
-  	strcpy(scr_passwd, p);
+  	strncpy(scr_user, l, sizeof(scr_user));
+  	strncpy(scr_name, s, sizeof(scr_name));
+  	strncpy(scr_passwd, p, sizeof(scr_passwd));
   }
+  sprintf(scr_lines, "%d", (int) lines);  /* jl 13.09.97 */
 
   /* Throw away status line if temporary */
   if (tempst) {
@@ -450,11 +570,11 @@ char *s, *l, *p;
 
   if (mcd(P_SCRIPTDIR) < 0) return;
 
-  sprintf(cmdline, "%s %s", P_SCRIPTPROG, scr_name);
+  snprintf(cmdline, sizeof(cmdline), "%s %s", P_SCRIPTPROG, scr_name);
 
   switch(udpid = fork()) {
   	case -1:
-  		werror("Out of memory: could not fork()");
+  		werror(_("Out of memory: could not fork()"));
   		close(pipefd[0]);
   		close(pipefd[1]);
   		(void) mcd("");
@@ -473,6 +593,7 @@ char *s, *l, *p;
   		setuid((uid_t)real_uid);
   		mc_setenv("LOGIN", scr_user);
   		mc_setenv("PASS", scr_passwd);
+		mc_setenv("TERMLIN", scr_lines);	/* jl 13.09.97 */
   		(void) fastexec(translate(cmdline));
   		exit(1);
   	default: /* Parent */
