@@ -10,11 +10,15 @@
  *		modify it under the terms of the GNU General Public License
  *		as published by the Free Software Foundation; either version
  *		2 of the License, or (at your option) any later version.
+ *
+ * 01.01.98 dickey@clark.net: fix for a history window closing bug
+ * fmg 8/20/97: Added support for Search of History Buffer
  */
 
 #include "port.h"
 #include "window.h"
 #include "charmap.h"
+#include "intl.h"
 
 /* Status line code on/off. */
 #define ST_LINE 1
@@ -100,7 +104,7 @@ extern WIN *us;
 
 int useattr = 1;
 int dirflush = 1;
-int LINES, COLS;
+extern int LINES, COLS;
 int usecolor = 0;
 WIN *stdwin;
 char *_tptr = CNULL;
@@ -127,7 +131,7 @@ int a1, a2, a3, a4;
 {
   char lala[80];
 
-  sprintf(lala, s, a1, a2, a3, a4);
+  snprintf(lala, sizeof(lala), s, a1, a2, a3, a4);
   write(2, lala, strlen(lala));
   return(0);
 }
@@ -353,7 +357,10 @@ char attr, color;
      if (doit != 0) {
   	_gotoxy(x, y);
   	_setattr(attr, color);
-	(void) outchar((screen_ibmpc || (attr & XA_ALTCHARSET)) ? c : wcharmap[(unsigned char)c]);
+	(void) outchar((screen_ibmpc || screen_iso || (attr & XA_ALTCHARSET)) ? c : wcharmap[(unsigned char)c]); 
+	/* a little test at 10.3.1998 / jl */
+	/* (void) outchar((unsigned char) c); */
+
 	curx++;
      }
      if (doit >= 0) {
@@ -455,7 +462,7 @@ int doclr;
   if ((w = (WIN *)malloc(sizeof(WIN))) == (WIN *)0) return(w);
   
   offs = (border != BNONE);
-  if (!screen_ibmpc && AS)
+  if (!screen_ibmpc && AS) 
 	xattr = attr | XA_ALTCHARSET;
   else
 	xattr = attr;
@@ -607,10 +614,10 @@ int replace;
 	_setattr(win->o_attr, win->o_color);
   }
   free(win->map);
-  free(win);
 #if HISTORY
   if (win->histbuf) free(win->histbuf);
 #endif
+  free(win);	/* 1.1.98 dickey@clark.net  */
   wflush();
 }
 
@@ -1019,13 +1026,69 @@ ELM *e;
   int x;
 
   /* MARK updated 02/17/94 - Fixes bug, to do all 80 cols, not 79 cols */
-  for(x = w->x1; x <= w->x2; x++) {
-	_write(e->value, w->direct, x,
-		y + w->y1, e->attr, e->color);
+  for(x = w->x1; x <= w->x2; x++)
+  {
+	_write(e->value, w->direct, x, y + w->y1, e->attr, e->color);
+		/*y + w->y1, XA_NORMAL, e->color);*/
 	e++;
   }
 }
 
+#if _SEARCH_HISTORY
+/*
+ * fmg 8/20/97
+ * 'accumulate' one line of ELM's into a string
+ * WHY: need this in search function to see if line contains search pattern
+ */
+void wdrawelm_var(w, y, e, buf)
+WIN *w;
+int y;
+ELM *e;
+char *buf;
+{
+  int x,c=0;
+
+  /* MARK updated 02/17/94 - Fixes bug, to do all 80 cols, not 79 cols */
+  for(x = w->x1; x <= w->x2; x++)
+  {
+	buf[c++]=e->value;
+	e++;
+  }
+}
+
+/*
+ * fmg 8/20/97
+ * 'draw' one line of ELM's in a window INVERTED (text-mode-wise)
+ * WHY: need this in search function to see if line contains search pattern
+ */
+void wdrawelm_inverse(w, y, e)
+WIN *w;
+int y;
+ELM *e;
+{
+  int x;
+
+  /* MARK updated 02/17/94 - Fixes bug, to do all 80 cols, not 79 cols */
+  /* filipg 8/19/97: this will BOLD-up the line */
+  /* first position */
+  x = w->x1;
+  _write(e->value, w->direct, x, y + w->y1, XA_NORMAL, e->color);
+
+  e++;
+
+  /* everything in the middle will be BLINK */
+  for(x = (w->x1+1); x <= (w->x2-1); x++)
+  {
+	_write(e->value, w->direct, x, y + w->y1, XA_BOLD, WHITE);
+	e++;
+  }
+
+  /* last position */
+  x = w->x2;
+  _write(e->value, w->direct, x, y + w->y1, XA_NORMAL, e->color);
+}
+#endif /*SEARCH_HISTORY*/
+
 /*
  * Print a string in a window.
  */
@@ -1055,7 +1118,7 @@ char *s, *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8, *a9, *a10;
   t = buf;
 
   _intern = 1;
-  sprintf(buf, s, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10);
+  snprintf(buf, sizeof(buf), s, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10);
   while(*t) wputc(win, *t++);
   _intern = 0;
   if (dirflush && win->direct) wflush();
@@ -1185,7 +1248,7 @@ int attr, fg, bg;
 
   while(*a != CNULL) {
   	count++;
-  	if (strlen(*a) > len) len = strlen(*a);
+  	if (strlen(_(*a)) > len) len = strlen(_(*a));
   	a++;
   }
   if (title != CNULL && strlen(title) + 2 > len) len = strlen(title) + 2;
@@ -1201,7 +1264,7 @@ int attr, fg, bg;
   if (title != CNULL) wtitle(w, TMID, title);
 
   for(c = 0; c < count; c++) {
-  	wprintf(w, " %s%s", choices[c], c == count - 1 ? "" : "\n");
+  	wprintf(w, " %s%s", _(choices[c]), c == count - 1 ? "" : "\n");
   }
 
   wcurbar(w, cur, high_on);
@@ -1209,7 +1272,8 @@ int attr, fg, bg;
 
   while (1) {
 	while((c = wxgetch()) != 27 && c != '\n' && c!= '\r' && c != ' ') {
-		if (c == K_UP || c == K_DN || c == 'j' || c == 'k')
+		if (c == K_UP || c == K_DN || c == 'j' || c == 'k' ||
+		    c == K_HOME || c == K_END)
 			wcurbar(w, cur, high_off);
 		switch(c) {
 			case K_UP:
@@ -1222,8 +1286,15 @@ int attr, fg, bg;
 				cur++;
 				if (cur >= count) cur = 0;
 				break;
+			case K_HOME:
+			  	cur = 0;
+				break;
+			case K_END:
+			        cur = count - 1;
+				break;
 		}
-		if (c == K_UP || c == K_DN || c == 'j' || c == 'k')
+		if (c == K_UP || c == K_DN || c == 'j' || c == 'k' ||
+		    c == K_HOME || c == K_END)
 			wcurbar(w, cur, high_on);
 	}
 	wcursor(w, CNORMAL);
@@ -1600,7 +1671,7 @@ int maxlen;
   /* We assume the line has already been drawn on the screen. */
   if ((idx = strlen(s)) > linelen)
 	idx = linelen;
-  strcpy(buf, s);
+  strncpy(buf, s, sizeof(buf));
   wlocate(w, x + idx, y);
   dirflush = 0;
   wflush();
@@ -1874,10 +1945,11 @@ int attr;
   
   _has_am = tgetflag("am");
   _mv_standout = tgetflag("ms");
-  if (tgetflag("bs"))
-	if (BC == CNULL) BC = "\b";
+  if (tgetflag("bs")) {
+    if (BC == CNULL) BC = "\b";
+  } 
   else
-  	BC = CNULL;	
+    BC = CNULL;	
 
   /* Special IBM box-drawing characters */
   D_UL  = 201;
@@ -1888,7 +1960,7 @@ int attr;
   D_LR  = 188;
   
   S_UL  = 218;
-  S_HOR = 196;
+  S_HOR = 240;
   S_UR  = 191;
   S_LL  = 192;
   S_VER = 179;
@@ -1904,6 +1976,17 @@ int attr;
 	D_LR  = S_LR  = acmap('j');
   }
 
+  if (screen_iso) {
+        /* Try to find AC mappings. */
+	D_UL  = S_UL  = '+';
+	D_HOR = S_HOR = '-';
+	D_UR  = S_UR  = '+';
+	D_LL  = S_LL  = '+';
+	D_VER = S_VER = '|';
+	D_LR  = S_LR  = '+';
+  }
+
+
 #if VERY_OLD
   if ((p = tgetstr("gA", &_tptr)) != CNULL) wcharmap[201] = *p;
   if ((p = tgetstr("gB", &_tptr)) != CNULL) wcharmap[205] = *p;
@@ -1912,7 +1995,7 @@ int attr;
   if ((p = tgetstr("gE", &_tptr)) != CNULL) wcharmap[186] = *p;
   if ((p = tgetstr("gF", &_tptr)) != CNULL) wcharmap[188] = *p;
   if ((p = tgetstr("gG", &_tptr)) != CNULL) wcharmap[218] = *p;
-  if ((p = tgetstr("gH", &_tptr)) != CNULL) wcharmap[196] = *p;
+  if ((p = tgetstr("gH", &_tptr)) != CNULL) wcharmap[240] = *p;
   if ((p = tgetstr("gI", &_tptr)) != CNULL) wcharmap[191] = *p;
   if ((p = tgetstr("gJ", &_tptr)) != CNULL) wcharmap[192] = *p;
   if ((p = tgetstr("gK", &_tptr)) != CNULL) wcharmap[179] = *p;
